# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos_tests import StageOneReportTest, redhat_only, ubuntu_only

# known values in our CI test images
FOREMAN_DB_PASSWORD = r'S0Sdb=p@ssw0rd!'
FOREMAN_ADMIN_PASSWORD = r'S0S@dmin\\p@ssw0rd!'
CANDLEPIN_DB_PASSWORD = r'S0SKatello%sp@ssw0rd!'

FOREMAN_PASSWORDS = [FOREMAN_DB_PASSWORD, FOREMAN_ADMIN_PASSWORD, CANDLEPIN_DB_PASSWORD]


class ForemanBasicTest(StageOneReportTest):
    """Ensure that a basic execution runs as expected with all TFM related
    plugins. For the Red Hat tests, it assumes Foreman has been deployed on a
    Katello system. On Debian systems, these tests are skipped.

    :avocado: tags=foreman
    """

    sos_cmd = '-v'

    def test_tfm_plugins_ran(self):
        self.assertPluginIncluded([
            'apache',
            'foreman',
            'foreman_installer',
            'foreman_proxy',
            'postgresql',
            'puppet',
            'ruby'
        ])

    @redhat_only
    def test_candlepin_plugin_ran(self):
        self.assertPluginIncluded('candlepin')

    def test_foreman_keys_skipped(self):
        self.assertFileGlobNotInArchive("/etc/foreman*/*key.pem")

    def test_foreman_database_sizes_collected(self):
        self.assertFileCollected('sos_commands/foreman/foreman_db_tables_sizes')

    def test_foreman_installer_dirs_collected(self):
        self.assertFileGlobInArchive("/etc/foreman-installer/*")
        self.assertFileGlobInArchive("/var/log/foreman-installer/*")

    def test_foreman_production_log_collected(self):
        self.assertFileCollected('/var/log/foreman/production.log')

    def test_foreman_database_dump_collected(self):
        self.assertFileCollected('sos_commands/foreman/foreman_settings_table')

    def test_foreman_tasks_csv_collected(self):
        self.assertFileCollected('sos_commands/foreman/foreman_tasks_tasks')

    def test_proxyfeatures_not_collected(self):
        self.assertFileGlobNotInArchive("sos_commands/foreman/smart_proxies_features/*")

    def test_foreman_config_postproc_worked(self):
        self.assertFileNotHasContent('/etc/foreman/database.yml', FOREMAN_DB_PASSWORD)

    def test_foreman_password_postproc_worked(self):
        for _check in ['/var/log/foreman-installer/foreman.log', '/etc/foreman-installer/scenarios.d/foreman-answers.yaml']:
            for passwd in FOREMAN_PASSWORDS:
                self.assertFileNotHasContent(_check, passwd)

    @redhat_only
    def test_candlepin_table_sizes_collected(self):
        self.assertFileCollected('sos_commands/candlepin/candlepin_db_tables_sizes')

    @redhat_only
    def test_katello_password_postproc_worked(self):
        for _check in ['/var/log/foreman-installer/katello.log', '/etc/foreman-installer/scenarios.d/katello-answers.yaml']:
            for passwd in FOREMAN_PASSWORDS:
                self.assertFileNotHasContent(_check, passwd)

    @redhat_only
    def test_foreman_httpd_collected(self):
        self.assertFileGlobInArchive("/var/log/httpd*/foreman-ssl_*_ssl*log*")

    @ubuntu_only
    def test_foreman_apache_collected(self):
        self.assertFileGlobInArchive("/var/log/apache2/foreman-ssl_*_ssl*log*")

    def test_ruby_gems_collected(self):
        self.assertFileCollected('sos_commands/ruby/gem_list')


class ForemanWithOptionsTest(StageOneReportTest):
    """Enable Foreman/Katello/Candlepin specific options and ensure they are
    working

    :avocado: tags=foreman
    """

    sos_cmd = '-v -k foreman.proxyfeatures=on'

    @redhat_only
    def test_proxyfeatures_collected(self):
        self.assertFileGlobInArchive("sos_commands/foreman/smart_proxies_features/*")

class ForemanInstallerTest(StageOneReportTest):
    """Check whether foreman-installer related data are properly collected
    independently on main foreman plugin.

    :avocado: tags=foreman
    """

    sos_cmd = '-v -o foreman_installer'

    def test_foreman_installer_etc_collected(self):
        self.assertFileCollected("/etc/foreman-installer/scenarios.d")

class ForemanProxyTest(StageOneReportTest):
    """Check whether foreman-proxy related data are properly collected
    independently on main foreman plugin.

    :avocado: tags=foreman
    """

    sos_cmd = '-v -o foreman_proxy'

    def test_foreman_proxy_settings_collected(self):
        self.assertFileCollected("/etc/foreman-proxy/settings.yml")



# Copyright 2012 Red Hat Inc.
# Guy Streeter <streeter@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class KernelRT(Plugin, RedHatPlugin):

    short_desc = 'Realtime kernel variant'

    plugin_name = 'kernelrt'
    profiles = ('system', 'hardware', 'kernel', 'mrg')

    # this file exists only when the realtime kernel is booted
    # this plugin will not be called is this file does not exist
    files = ('/sys/kernel/realtime',)

    def setup(self):
        clocksource_path = '/sys/devices/system/clocksource/clocksource0/'
        self.add_copy_spec([
            '/etc/rtgroups',
            '/proc/sys/kernel/sched_rt_period_us',
            '/proc/sys/kernel/sched_rt_runtime_us',
            '/sys/kernel/realtime',
            clocksource_path + 'available_clocksource',
            clocksource_path + 'current_clocksource'
        ])
        # note: rhbz#1059685 'tuna - NameError: global name 'cgroups' is not
        # defined this command throws an exception on versions prior to
        # 0.10.4-5.
        self.add_cmd_output('tuna -CP')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Block(Plugin, IndependentPlugin):

    short_desc = 'Block device information'

    plugin_name = 'block'
    profiles = ('storage', 'hardware')
    verify_packages = ('util-linux',)
    files = ('/sys/block',)

    def setup(self):
        self.add_forbidden_path("/sys/block/*/queue/iosched")

        self.add_file_tags({
            '/sys/block/.*/queue/scheduler': 'scheduler'
        })

        self.add_cmd_output([
            "lsblk",
            "lsblk -t",
            "lsblk -D",
            "blkid -c /dev/null",
            "blockdev --report",
            "ls -lanR /dev",
            "ls -lanR /sys/block",
            "lsblk -O -P",
            "losetup -a",
        ])

        # legacy location for non-/run distributions
        self.add_copy_spec([
            "/etc/blkid.tab",
            "/run/blkid/blkid.tab",
            "/proc/partitions",
            "/proc/diskstats",
            "/sys/block/*/queue/",
            "/sys/block/sd*/device/timeout",
            "/sys/block/hd*/device/timeout",
            "/sys/block/sd*/device/state",
            "/sys/block/loop*/loop/",
        ])

        cmds = [
            "parted -s %(dev)s unit s print",
            "fdisk -l %(dev)s",
            "udevadm info %(dev)s",
            "udevadm info -a %(dev)s"
        ]
        self.add_device_cmd(cmds, devices='block', blacklist='ram.*')

        lsblk = self.collect_cmd_output("lsblk -f -a -l")
        # for LUKS devices, collect cryptsetup luksDump
        if lsblk['status'] == 0:
            for line in lsblk['output'].splitlines():
                if 'crypto_LUKS' in line:
                    dev = line.split()[0]
                    self.add_cmd_output('cryptsetup luksDump /dev/%s' % dev)

# vim: set et ts=4 sw=4 :



# -*- python -*-
# -*- coding: utf-8 -*-

# Copyright (C) 2015 Red Hat, Inc., Pep Turró Mauri <pep@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin
from os import environ
import os.path


class ManageIQ(Plugin, RedHatPlugin):

    short_desc = 'ManageIQ/CloudForms related information'

    plugin_name = 'manageiq'

    miq_dir = '/var/www/miq/vmdb'

    packages = (
        'cfme',
        'cfme-appliance',
        'cfme-gemset',
        'cfme-appliance-tools',
        'cfme-appliance-common'
    )
    files = (
        os.path.join(miq_dir, 'BUILD'),
        os.path.join(miq_dir, 'GUID'),
        os.path.join(miq_dir, 'VERSION'),
        os.path.join(miq_dir, 'REGION')
    )

    # Config files to collect from miq_dir/config/
    miq_conf_dir = os.path.join(miq_dir, "config")
    miq_conf_files = [
        '*.rb',
        '*.yaml',
        '*.yml',
        '*.yml.db',
        '*.yml.sample',
        'settings/*.yml',
        'environments/*.rb',
        'environments/*.yml',
        'environments/patches/*.rb',
        'initializers/*.rb',
        'database.yml.old',
        'brakeman.ignore',
    ]

    # Log files to collect from miq_dir/log/
    miq_log_dir = os.path.join(miq_dir, "log")

    miq_main_logs = [
        'ansible_tower.log',
        'top_output.log',
        'evm.log',
        'production.log',
        'automation.log',
    ]

    miq_log_files = [
        '*.log',
        'apache/*.log',
        '*.txt',
        '*.yml',
    ]

    def setup(self):
        if self.get_option("all_logs"):
            # turn all log files to a glob to include logrotated ones
            self.miq_log_files = map(lambda x: x + '*', self.miq_log_files)

        self.add_copy_spec(list(self.files))

        self.add_copy_spec([
            self.path_join(self.miq_conf_dir, x) for x in self.miq_conf_files
        ])

        # Collect main log files without size limit.
        self.add_copy_spec([
            self.path_join(self.miq_log_dir, x) for x in self.miq_main_logs
        ], sizelimit=0)

        self.add_copy_spec([
            self.path_join(self.miq_log_dir, x) for x in self.miq_log_files
        ])

        self.add_copy_spec([
            "/var/log/tower.log",
            "/etc/manageiq/postgresql.conf.d/*.conf"
        ])

        if environ.get("APPLIANCE_PG_DATA"):
            pg_dir = environ.get("APPLIANCE_PG_DATA")
            self.add_copy_spec([
                    self.path_join(pg_dir, 'pg_log'),
                    self.path_join(pg_dir, 'postgresql.conf')
            ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Login(Plugin, IndependentPlugin):

    short_desc = 'login information'

    plugin_name = 'login'
    profiles = ('system', 'identity')

    def setup(self):
        self.add_cmd_output("last", root_symlink="last")
        self.add_cmd_output([
            "last reboot",
            "last shutdown",
            "lastlog",
            "lastlog -u 0-999",
            "lastlog -u 1000-60000",
            "lastlog -u 60001-65536",
            "lastlog -u 65537-4294967295"
        ])

        self.add_copy_spec([
            "/etc/login.defs",
            "/etc/default/useradd",
        ])

# vim: et ts=4 sw=4



# Copyright (C) 2016 Red Hat, Inc., Pratik Bandarkar <pbandark@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Grafana(Plugin, IndependentPlugin):

    short_desc = 'Fetch Grafana configuration, logs and CLI output'
    plugin_name = "grafana"
    profiles = ('services', 'openstack', 'openstack_controller')

    packages = ('grafana',)

    def setup(self):
        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/grafana/*.log*")
        else:
            self.add_copy_spec("/var/log/grafana/*.log")

        self.add_cmd_output([
            "grafana-cli plugins ls",
            "grafana-cli plugins list-remote",
            "grafana-cli -v",
            "grafana-server -v",
        ])

        self.add_copy_spec([
            "/etc/grafana/",
            "/etc/sysconfig/grafana-server",
        ])

    def postproc(self):
        protect_keys = [
            "admin_password", "secret_key"
        ]

        regexp = r"((?m)^\s*(%s)\s*=\s*)(.*)" % "|".join(protect_keys)
        self.do_path_regex_sub("/etc/grafana/grafana.ini",
                               regexp, r"\1*********")



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)
import re
import os
from datetime import datetime as dt


class Sar(Plugin):
    """
    The sar plugin is designed to collect system performance data as recorded
    by sysstat.

    The raw binary data, i.e. the 'saX' files, will be collected and for files
    a week old or younger, this plugin will capture human-readable conversions
    of those files provided by the 'sar' command locally available, if the
    local sysstat installation has not already created a converted copy (e.g.
    for the current day-of data being collected at the time of report
    generation).

    Using the 'all_sar' plugin option will not only cause the plugin to capture
    _all_ 'saX' files present on the host, but further perform the 'sar'
    conversion on all files, not just those produced within the last week.

    Converted 'sar' files will be written to the sos_commands/sar/, and not
    to the /var/log/ path that sysstat writes to.

    Note that this conversion is done because it is unlikely that the same
    version of sysstat that produces the 'saX' files will be the same version
    available on a given analyst's workstation, and this conversion is version
    sensitive.
    """

    short_desc = 'System Activity Reporter'

    plugin_name = 'sar'
    profiles = ('system', 'performance')

    packages = ('sysstat',)
    sa_path = '/var/log/sa'
    option_list = [
        PluginOpt('all_sar', default=False,
                  desc="gather all system activity records")
    ]

    def setup(self):
        self.add_copy_spec(self.path_join(self.sa_path, '*'),
                           sizelimit=0 if self.get_option("all_sar") else None,
                           tailit=False)

        try:
            dir_list = self.listdir(self.sa_path)
        except OSError:
            self._log_warn("sar: could not list %s" % self.sa_path)
            return
        sa_regex = re.compile(r"sa[\d]+")
        # find all the sa files that don't have an existing sar file
        # there are two possible formats for sar files
        # saDD, the default one where DD is the day of the month
        # saYYYYMMDD, which is the format when specifying -D
        # as option for sadc
        for fname in dir_list:
            if sa_regex.match(fname):
                sa_data_path = self.path_join(self.sa_path, fname)
                sar_filename = 'sar' + fname[2:]
                if sar_filename not in dir_list:
                    # only collect sar output for the last 7 days by default
                    if not self.get_option('all_sar'):
                        try:
                            _ftime = os.stat(sa_data_path).st_mtime
                            _age = dt.today() - dt.fromtimestamp(_ftime)
                            if _age.days > 7:
                                continue
                        except Exception as err:
                            self._log_warn(
                                "Could not determine age of '%s' - skipping "
                                "converting to sar format: %s"
                                % (sa_data_path, err)
                            )
                            continue
                    sar_cmd = "sar -A -f %s" % sa_data_path
                    self.add_cmd_output(sar_cmd, sar_filename)
                sadf_cmd = "sadf -x -- -A %s" % sa_data_path
                self.add_cmd_output(sadf_cmd, "%s.xml" % fname)


class RedHatSar(Sar, RedHatPlugin):

    sa_path = '/var/log/sa'


class DebianSar(Sar, DebianPlugin, UbuntuPlugin):

    sa_path = '/var/log/sysstat'

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Boom(Plugin, RedHatPlugin):

    short_desc = 'Configuration data for the boom boot manager.'

    plugin_name = 'boom'
    profiles = ('boot', 'system')

    packages = (
        'lvm2-python-boom',
        'python-boom',
        'python2-boom',
        'python3-boom'
    )

    def setup(self):
        self.add_copy_spec([
            "/boot/boom",
            "/boot/loader/entries",
            "/etc/default/boom",
            "/etc/grub.d/42_boom"
        ])

        self.add_cmd_output([
            "boom list -VV",
            "boom profile list -VV"
        ])

# vim: set et ts=4 sw=4 :



# Copyright 2022 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.parsers import SoSCleanerParser
from sos.cleaner.mappings.ipv6_map import SoSIPv6Map


class SoSIPv6Parser(SoSCleanerParser):
    """Parser for handling IPv6 networks and addresses"""

    name = 'IPv6 Parser'
    map_file_key = 'ipv6_map'
    regex_patterns = [
        # Attention: note that this is a single long regex, not several entries
        # This is initially based off of two regexes from the Java library
        # for validating an IPv6 string. However, this is modified to begin and
        # end with a negative lookbehind to ensure that a substring of 'ed::'
        # is not extracted from a log message such as 'SomeFuncUsed::ADiffFunc'
        # that come components may log with. Further, we optionally try to grab
        # a trailing prefix for the network bits.
        r"(?<![:\\.\\-a-z0-9])((([0-9a-f]{1,4})(:[0-9a-f]{1,4}){7})|"
        r"(([0-9a-f]{1,4}(:[0-9a-f]{0,4}){0,5}))([^.])::(([0-9a-f]{1,4}"
        r"(:[0-9a-f]{1,4}){0,5})?))(/\d{1,3})?(?![:\\a-z0-9])"
    ]
    skip_files = [
        'etc/dnsmasq.conf.*',
        '.*modinfo.*',
    ]
    compile_regexes = False

    def __init__(self, config):
        self.mapping = SoSIPv6Map()
        super(SoSIPv6Parser, self).__init__(config)

    def get_map_contents(self):
        """Structure the dataset contents properly so that they can be reloaded
        on subsequent runs correctly.
        """
        _d = {
            'version': self.mapping.version,
            'networks': {}
        }
        for net in self.mapping.networks:
            _net = self.mapping.networks[net]
            _d['networks'][_net.original_address] = {
                'obfuscated': _net.obfuscated_address,
                'hosts': {}
            }
            for host in _net.hosts:
                _ob_host = _net.hosts[host]
                _d['networks'][_net.original_address]['hosts'][host] = _ob_host

        return _d



# Copyright (C) 2007 Red Hat, Inc., Kent Lamb <klamb@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, SoSPredicate
from glob import glob


class Ipa(Plugin, RedHatPlugin):

    short_desc = 'Identity, policy, audit'

    plugin_name = 'ipa'
    profiles = ('identity', 'apache')

    ipa_server = False
    ipa_client = False

    files = ('/etc/ipa',)
    packages = ('ipa-server', 'ipa-client', 'freeipa-server', 'freeipa-client')

    def check_ipa_server_version(self):
        if self.is_installed("pki-server") \
                or self.path_exists("/var/lib/pki") \
                or self.path_exists("/usr/share/doc/ipa-server-4.2.0"):
            return "v4"
        elif self.is_installed("pki-common") \
                or self.path_exists("/var/lib/pki-ca/"):
            return "v3"
        return None

    def ca_installed(self):
        # Follow the same checks as IPA CA installer code
        if self.path_exists("%s/conf/ca/CS.cfg" % self.pki_tomcat_dir_v4) \
                or self.path_exists("%s/conf/CS.cfg" % self.pki_tomcat_dir_v3):
            return True

    def ipa_server_installed(self):
        if self.is_installed("ipa-server") \
                or self.is_installed("freeipa-server"):
            return True

    def retrieve_pki_logs(self, ipa_version):
        if ipa_version == "v4":
            self.add_copy_spec([
               "/var/log/pki/pki-tomcat/ca/debug*",
               "/var/log/pki/pki-tomcat/ca/system",
               "/var/log/pki/pki-tomcat/ca/transactions",
               "/var/log/pki/pki-tomcat/ca/selftests.log",
               "/var/log/pki/pki-tomcat/catalina.*",
               "/var/log/pki/pki-ca-spawn.*",
               "/var/log/pki/pki-tomcat/kra/debug*",
               "/var/log/pki/pki-tomcat/kra/system",
               "/var/log/pki/pki-tomcat/kra/transactions",
               "/var/log/pki/pki-kra-spawn.*"
            ])
        elif ipa_version == "v3":
            self.add_copy_spec([
               "/var/log/pki-ca/debug",
               "/var/log/pki-ca/system",
               "/var/log/pki-ca/transactions",
               "/var/log/pki-ca/selftests.log",
               "/var/log/pki-ca/catalina.*",
               "/var/log/pki/pki-ca-spawn.*"
            ])

    def setup(self):
        self.pki_tomcat_dir_v4 = "/var/lib/pki/pki-tomcat"
        self.pki_tomcat_dir_v3 = "/var/lib/pki-ca"

        self.pki_tomcat_conf_dir_v4 = "/etc/pki/pki-tomcat/ca"
        self.pki_tomcat_conf_dir_v3 = "/etc/pki-ca"

        # Returns "v3", "v4", or None
        ipa_version = self.check_ipa_server_version()

        if self.ipa_server_installed():
            self._log_debug("IPA server install detected")

            self._log_debug("IPA version is [%s]" % ipa_version)

            self.add_copy_spec([
                "/var/log/ipaserver-install.log",
                "/var/log/ipaserver-kra-install.log",
                "/var/log/ipareplica-install.log",
                "/var/log/ipareplica-ca-install.log",
                "/var/log/ipa-custodia.audit.log"
            ])

        if self.ca_installed():
            self._log_debug("CA is installed: retrieving PKI logs")
            self.retrieve_pki_logs(ipa_version)

        self.add_copy_spec([
            "/var/log/ipaclient-install.log",
            "/var/log/ipaupgrade.log",
            "/var/log/krb5kdc.log",
            "/var/log/dirsrv/slapd-*/logs/access",
            "/var/log/dirsrv/slapd-*/logs/errors",
            "/etc/dirsrv/slapd-*/dse.ldif",
            "/etc/dirsrv/slapd-*/schema/99user.ldif",
            "/etc/hosts",
            "/etc/httpd/alias/*",
            "/etc/named.*",
            "/etc/ipa/ca.crt",
            "/etc/ipa/default.conf",
            "/etc/ipa/kdcproxy/kdcproxy.conf",
            "/etc/ipa/kdcproxy/ipa-kdc-proxy.conf",
            "/etc/ipa/kdcproxy.conf",
            "/root/.ipa/log/cli.log",
            "/var/lib/certmonger/requests/[0-9]*",
            "/var/lib/certmonger/cas/[0-9]*",
            "/var/lib/ipa/ra-agent.pem",
            "/var/lib/ipa/certs/httpd.crt",
            "/var/kerberos/krb5kdc/kdc.crt",
            "/var/lib/ipa/sysrestore/sysrestore.state",
            "/var/log/ipa/healthcheck/healthcheck.log*"
        ])

        #  Make sure to use the right PKI config and NSS DB folders
        if ipa_version == "v4":
            self.pki_tomcat_dir = self.pki_tomcat_dir_v4
            self.pki_tomcat_conf_dir = self.pki_tomcat_conf_dir_v4
        else:
            self.pki_tomcat_dir = self.pki_tomcat_dir_v3
            self.pki_tomcat_conf_dir = self.pki_tomcat_conf_dir_v3

        self.add_cmd_output("certutil -L -d %s/alias" % self.pki_tomcat_dir)
        self.add_copy_spec("%s/CS.cfg" % self.pki_tomcat_conf_dir)

        self.add_forbidden_path([
            "/etc/pki/nssdb/key*",
            "/etc/dirsrv/slapd-*/key*",
            "/etc/dirsrv/slapd-*/pin.txt",
            "/etc/dirsrv/slapd-*/pwdfile.txt",
            "/etc/httpd/alias/ipasession.key",
            "/etc/httpd/alias/key*",
            "/etc/httpd/alias/pin.txt",
            "/etc/httpd/alias/pwdfile.txt",
            "/etc/named.keytab",
            "%s/alias/key*" % self.pki_tomcat_dir,
            "%s/flatfile.txt" % self.pki_tomcat_conf_dir,
            "%s/password.conf" % self.pki_tomcat_conf_dir,
        ])

        self.add_cmd_output([
            "ls -la /etc/dirsrv/slapd-*/schema/",
            "certutil -L -d /etc/httpd/alias/",
            "pki-server cert-find --show-all",
            "pki-server subsystem-cert-validate ca",
            "klist -ket /etc/dirsrv/ds.keytab",
            "klist -ket /etc/httpd/conf/ipa.keytab",
            "klist -ket /var/lib/ipa/gssproxy/http.keytab"
        ])

        getcert_pred = SoSPredicate(self,
                                    services=['certmonger'])

        self.add_cmd_output("getcert list", pred=getcert_pred)

        for certdb_directory in glob("/etc/dirsrv/slapd-*/"):
            self.add_cmd_output("certutil -L -d %s" % certdb_directory)
        return

    def postproc(self):
        match = r"(\s*arg \"password )[^\"]*"
        subst = r"\1********"
        self.do_file_sub("/etc/named.conf", match, subst)

        self.do_cmd_output_sub("getcert list",
                               r"(pin=)'(\d+)'",
                               r"\1'***'")

        request_logs = "/var/lib/certmonger/requests/[0-9]*"
        for request_log in glob(request_logs):
            self.do_file_sub(request_log,
                             r"(key_pin=)(\d+)",
                             r"\1***")


# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Red Hat, Inc., Adam Stokes <astokes@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin, DebianPlugin


class Autofs(Plugin):

    short_desc = 'Autofs on-demand automounter'

    plugin_name = "autofs"
    profiles = ('storage', 'nfs')

    files = ('/etc/sysconfig/autofs', '/etc/default/autofs')
    packages = ('autofs',)

    def checkdebug(self):
        """ testing if autofs debug has been enabled anywhere
        """
        # Global debugging
        opt = self.file_grep(r"^(DEFAULT_LOGGING|DAEMONOPTIONS)=(.*)",
                             *self.files)
        for opt1 in opt:
            for opt2 in opt1.split(" "):
                if opt2 in ("--debug", "debug"):
                    return True
        return False

    def getdaemondebug(self):
        """ capture daemon debug output
        """
        debugout = self.file_grep(r"^(daemon.*)\s+(\/var\/log\/.*)",
                                  *self.files)
        for i in debugout:
            return i[1]

    def setup(self):
        self.add_copy_spec("/etc/auto*")
        self.add_service_status("autofs")
        self.add_cmd_output("automount -m")
        if self.checkdebug():
            self.add_copy_spec(self.getdaemondebug())

    def postproc(self):
        self.do_path_regex_sub(
            "/etc/auto*",
            r"(password=)[^,\s]*",
            r"\1********"
        )
        # Hide secrets in the LDAP authentication config
        #
        # Example of scrubbing of the secret:
        #
        #     secret="abc"
        #   or
        #     encoded_secret = 'abc'
        #
        # to:
        #
        #     secret="********"
        #   or
        #     encoded_secret = '********'
        #
        self.do_file_sub(
            "/etc/autofs_ldap_auth.conf",
            r"(secret[\s]*[=]+[\s]*)(\'|\").*(\'|\")",
            r"\1\2********\3"
        )
        self.do_cmd_output_sub(
            "automount -m",
            r"(password=)[^,\s]*",
            r"\1********"
        )


class RedHatAutofs(Autofs, RedHatPlugin):

    def setup(self):
        super(RedHatAutofs, self).setup()
        if self.get_option("verify"):
            self.add_cmd_output("rpm -qV autofs")


class DebianAutofs(Autofs, DebianPlugin, UbuntuPlugin):

    def setup(self):
        super(DebianAutofs, self).setup()
        self.add_cmd_output("dpkg-query -s autofs")

# vim: set et ts=4 sw=4 :



# Copyright 2022 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import ipaddress

from random import getrandbits
from sos.cleaner.mappings import SoSMap


def generate_hextets(hextets):
    """Generate a random set of hextets, based on the length of the source
    hextet. If any hextets are compressed, keep that compression.

    E.G. '::1234:bcd' will generate a leading empty '' hextet, followed by two
    4-character hextets.

    :param hextets:     The extracted hextets from a source address
    :type hextets:      ``list``

    :returns:           A set of randomized hextets for use in an obfuscated
                        address
    :rtype:             ``list``
    """
    return [random_hex(4) if h else '' for h in hextets]


def random_hex(length):
    """Generate a string of size length of random hex characters.

    :param length:  The number of characters to generate
    :type length:   ``int``

    :returns:       A string of ``length`` hex characters
    :rtype:         ``str``
    """
    return f"{getrandbits(4*length):0{length}x}"


class SoSIPv6Map(SoSMap):
    """Mapping for IPv6 addresses and networks.

    Much like the IP map handles IPv4 addresses, this map is designed to take
    IPv6 strings and obfuscate them consistently to maintain network topology.
    To do this, addresses will be manipulated by the ipaddress library.

    If an IPv6 address is encountered without a netmask, it is assumed to be a
    /64 address.
    """

    networks = {}

    ignore_matches = [
        r'^::1/.*',
        r'::/0',
        r'fd53:.*',
        r'^53..:'
    ]

    first_hexes = ['534f']

    compile_regexes = False
    version = 1

    def conf_update(self, config):
        """Override the base conf_update() so that we can load the existing
        networks into ObfuscatedIPv6Network() objects for the current run.
        """
        if 'networks' not in config:
            return
        for network in config['networks']:
            _orig = ipaddress.ip_network(network)
            _obfuscated = config['networks'][network]['obfuscated']
            _net = self._get_network(_orig, _obfuscated)
            self.dataset[_net.original_address] = _net.obfuscated_address
            for host in config['networks'][network]['hosts']:
                _ob_host = config['networks'][network]['hosts'][host]
                _net.add_obfuscated_host_address(host, _ob_host)
                self.dataset[host] = _ob_host

    def sanitize_item(self, ipaddr):
        _prefix = ipaddr.split('/')[-1] if '/' in ipaddr else ''
        _ipaddr = ipaddr
        if not _prefix:
            # assume a /64 default per protocol
            _ipaddr += "/64"
        try:
            _addr = ipaddress.ip_network(_ipaddr)
            # ipaddr was an actual network per protocol
            _net = self._get_network(_addr)
            _ipaddr = _net.obfuscated_address
        except ValueError:
            # A ValueError is raised from the ipaddress module when passing
            # an address such as 2620:52:0:2d80::4fe/64, which has host bits
            # '::4fe' set - the /64 is generally interpreted only for network
            # addresses. We use this behavior to properly obfuscate the network
            # before obfuscating a host address within that network
            _addr = ipaddress.ip_network(_ipaddr, strict=False)
            _net = self._get_network(_addr)
            if _net.network_addr not in self.dataset:
                self.dataset[_net.original_address] = _net.obfuscated_address
            # then, get the address within the network
            _hostaddr = ipaddress.ip_address(_ipaddr.split('/')[0])
            _ipaddr = _net.obfuscate_host_address(_hostaddr)

        if _prefix and '/' not in _ipaddr:
            return f"{_ipaddr}/{_prefix}"
        return _ipaddr

    def _get_network(self, address, obfuscated=''):
        """Attempt to find an existing ObfuscatedIPv6Network object from which
        to either find an existing obfuscated match, or create a new one. If
        no such object already exists, create it.
        """
        _addr = address.compressed
        if _addr not in self.networks:
            self.networks[_addr] = ObfuscatedIPv6Network(address, obfuscated,
                                                         self.first_hexes)
        return self.networks[_addr]


class ObfuscatedIPv6Network():
    """An abstraction class that represents a network that is (to be) handled
    by sos.

    Each distinct IPv6 network that we encounter will have a representative
    instance of this class, from which new obfuscated subnets and host
    addresses will be generated.

    This class should be built from an ``ipaddress.IPv6Network`` object. If
    an obfuscation string is not passed, one will be created during init.
    """

    def __init__(self, addr, obfuscation='', used_hexes=None):
        """Basic setup for the obfuscated network. Minor validation on the addr
        used to create the instance, as well as on an optional ``obfuscation``
        which if set, will serve as the obfuscated_network address.

        :param addr:        The *un*obfuscated network to be handled
        :type addr:         ``ipaddress.IPv6Network``

        :param obfuscation: An optional pre-determined string representation of
                            the obfuscated network address
        :type obfuscation:  ``str``

        :param used_hexes:  A list of already used hexes for the first hextet
                            of a potential global address obfuscation
        :type used_hexes:   ``list``
        """
        if not isinstance(addr, ipaddress.IPv6Network):
            raise Exception('Invalid network: not an IPv6Network object')
        self.addr = addr
        self.prefix = addr.prefixlen
        self.network_addr = addr.network_address.compressed
        self.hosts = {}
        if used_hexes is None:
            self.first_hexes = ['534f']
        else:
            self.first_hexes = used_hexes
        if not obfuscation:
            self._obfuscated_network = self._obfuscate_network_address()
        else:
            if not isinstance(obfuscation, str):
                raise TypeError(f"Pre-determined obfuscated network address "
                                f"must be str, not {type(obfuscation)}")
            self._obfuscated_network = obfuscation.split('/')[0]

    @property
    def obfuscated_address(self):
        return f"{self._obfuscated_network}/{self.prefix}"

    @property
    def original_address(self):
        return self.addr.compressed

    def _obfuscate_network_address(self):
        """Generate the obfuscated pair for the network address. This is
        determined based on the netmask of the network this class was built
        on top of.
        """
        if self.addr.is_global:
            return self._obfuscate_global_address()
        elif self.addr.is_link_local:
            # link-local addresses are always fe80::/64. This is not sensitive
            # in itself, and retaining the information that an address is a
            # link-local address is important for problem analysis, so don't
            # obfuscate this network information.
            return self.network_addr
        elif self.addr.is_private:
            return self._obfuscate_private_address()
        return self.network_addr

    def _obfuscate_global_address(self):
        """Global unicast addresses have a 48-bit global routing prefix and a
        16-bit subnet. We set the global routing prefix to a static
        sos-specific identifier that could never be seen in the wild,
        '534f:'

        We then randomize the subnet hextet.
        """
        _hextets = self.network_addr.split(':')[1:]
        _ob_hex = ['534f']
        if all(not c for c in _hextets):
            # we have only a single defined hextet, e.g. ff00::/64, so we need
            # to not use the standard first-hex identifier or we'll overlap
            # every similar address obfuscation.
            # Set the leading bits to 53, but increment upwards from there for
            # when we exceed 256 networks obfuscated in this manner.
            _start = 53 + (len(self.first_hexes) // 256)
            _ob_hex = f"{_start}{random_hex(2)}"
            while _ob_hex in self.first_hexes:
                # prevent duplicates
                _ob_hex = f"{_start}{random_hex(2)}"
            self.first_hexes.append(_ob_hex)
            _ob_hex = [_ob_hex]
        _ob_hex.extend(generate_hextets(_hextets))
        return ':'.join(_ob_hex)

    def _obfuscate_private_address(self):
        """The first 8 bits will always be 'fd', the next 40 bits are meant
        to be a global ID, followed by 16 bits for the subnet. To keep things
        relatively simply we maintain the first hextet as 'fd53', and then
        randomize any remaining hextets
        """
        _hextets = self.network_addr.split(':')[1:]
        _ob_hex = ['fd53']
        _ob_hex.extend(generate_hextets(_hextets))
        return ':'.join(_ob_hex)

    def obfuscate_host_address(self, addr):
        """Given an unobfuscated address, generate an obfuscated match for it,
        and save it to this network for tracking during the execution of clean.

        Note: another way to do this would be to convert the obfuscated network
        to bytes, and add a random amount to that based on the number of
        addresses that the network can support and from that new bytes count
        craft a new IPv6 address. This has the advantage of absolutely
        guaranteeing the new address is within the network space (whereas the
        method employed below could *theoretically* generate an overlapping
        address), but would in turn remove any ability to compress obfuscated
        addresses to match the general format/syntax of the address it is
        replacing. For the moment, it is assumed that being able to maintain a
        quick mental note of "unobfuscated device ff00::1 is obfuscated device
        53ad::a1b2" is more desireable than "ff00::1 is now obfuscated as
        53ad::1234:abcd:9876:a1b2:".

        :param addr:        The unobfuscated IPv6 address
        :type addr:         ``ipaddress.IPv6Address``

        :returns:           An obfuscated address within this network
        :rtype:             ``str``
        """
        def _generate_address():
            return ''.join([
                self._obfuscated_network,
                ':'.join(generate_hextets(_host.split(':')))
            ])

        if addr.compressed not in self.hosts:
            # separate host from the address by removing its network prefix
            _n = self.network_addr.rstrip(':')
            _host = addr.compressed[len(_n):].lstrip(':')
            _ob_host = _generate_address()
            while _ob_host in self.hosts.values():
                _ob_host = _generate_address()
            self.add_obfuscated_host_address(addr.compressed, _ob_host)
        return self.hosts[addr.compressed]

    def add_obfuscated_host_address(self, host, obfuscated):
        """Adds an obfuscated pair to the class for tracking and ongoing
        consistency in obfuscation.
        """
        self.hosts[host] = obfuscated



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os

from sos.cleaner.parsers import SoSCleanerParser
from sos.cleaner.mappings.keyword_map import SoSKeywordMap


class SoSKeywordParser(SoSCleanerParser):
    """Handles parsing for user provided keywords
    """

    name = 'Keyword Parser'
    map_file_key = 'keyword_map'

    def __init__(self, config, keywords=None, keyword_file=None):
        self.mapping = SoSKeywordMap()
        self.user_keywords = []
        super(SoSKeywordParser, self).__init__(config)
        for _keyword in self.mapping.dataset.keys():
            self.user_keywords.append(_keyword)
        if keywords:
            for keyword in keywords:
                if keyword not in self.user_keywords:
                    # pre-generate an obfuscation mapping for each keyword
                    # this is necessary for cases where filenames are being
                    # obfuscated before or instead of file content
                    self.mapping.get(keyword.lower())
                    self.user_keywords.append(keyword)
        if keyword_file and os.path.exists(keyword_file):
            with open(keyword_file, 'r') as kwf:
                self.user_keywords.extend(kwf.read().splitlines())

    def _parse_line(self, line):
        return line, 0



# Copyright (C) 2015 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class LightDm(Plugin, IndependentPlugin):

    short_desc = 'Light Display Manager'
    packages = ('lightdm', )
    profiles = ('desktop', )
    services = ('lightdm', )
    plugin_name = 'lightdm'

    def setup(self):
        self.add_copy_spec([
            "/etc/lightdm/lightdm.conf",
            "/etc/lightdm/users.conf"
        ])
        if not self.get_option("all_logs"):
            self.add_copy_spec("/var/log/lightdm/lightdm.log")
            self.add_copy_spec("/var/log/lightdm/x-0-greeter.log")
            self.add_copy_spec("/var/log/lightdm/x-0.log")
        else:
            self.add_copy_spec("/var/log/lightdm")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Michele Baldessari <michele at acksyn.org>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, PluginOpt
import os
from socket import gethostname


class Pcp(Plugin, RedHatPlugin, DebianPlugin):

    short_desc = 'Performance Co-Pilot data'

    plugin_name = 'pcp'
    profiles = ('system', 'performance')
    packages = ('pcp',)

    pcp_conffile = '/etc/pcp.conf'

    # size-limit of PCP logger and manager data collected by default (MB)
    option_list = [
        PluginOpt('pmmgrlogs', default=100,
                  desc='size limit in MB of pmmgr logs'),
        PluginOpt('pmloggerfiles', default=12,
                  desc='number of pmlogger files to collect')
    ]

    pcp_sysconf_dir = None
    pcp_var_dir = None
    pcp_log_dir = None

    pcp_hostname = ''

    def get_size(self, path):
        total_size = 0
        for dirpath, dirnames, filenames in os.walk(path):
            for f in filenames:
                fp = self.path_join(dirpath, f)
                total_size += os.path.getsize(fp)
        return total_size

    def pcp_parse_conffile(self):
        try:
            pcpconf = open(self.pcp_conffile, "r")
            lines = pcpconf.readlines()
            pcpconf.close()
        except IOError:
            return False
        env_vars = {}
        for line in lines:
            if line.startswith('#'):
                continue
            try:
                (key, value) = line.strip().split('=')
                env_vars[key] = value
            except (ValueError, KeyError):
                pass

        try:
            self.pcp_sysconf_dir = env_vars['PCP_SYSCONF_DIR']
            self.pcp_var_dir = env_vars['PCP_VAR_DIR']
            self.pcp_log_dir = env_vars['PCP_LOG_DIR']
        except Exception:
            # Fail if all three env variables are not found
            return False

        return True

    def setup(self):
        self.sizelimit = (None if self.get_option("all_logs")
                          else self.get_option("pmmgrlogs"))
        self.countlimit = (None if self.get_option("all_logs")
                           else self.get_option("pmloggerfiles"))

        if not self.pcp_parse_conffile():
            self._log_warn("could not parse %s" % self.pcp_conffile)
            return

        # Add PCP_SYSCONF_DIR (/etc/pcp) and PCP_VAR_DIR (/var/lib/pcp/config)
        # unconditionally. Obviously if someone messes up their /etc/pcp.conf
        # in a ridiculous way (i.e. setting PCP_SYSCONF_DIR to '/') this will
        # break badly.
        var_conf_dir = self.path_join(self.pcp_var_dir, 'config')
        self.add_copy_spec([
            self.pcp_sysconf_dir,
            self.pcp_conffile,
            var_conf_dir
        ])

        # We explicitly avoid /var/lib/pcp/config/{pmchart,pmlogconf,pmieconf,
        # pmlogrewrite} as in 99% of the cases they are just copies from the
        # rpms. It does not make up for a lot of size but it contains many
        # files
        self.add_forbidden_path([
            self.path_join(var_conf_dir, 'pmchart'),
            self.path_join(var_conf_dir, 'pmlogconf'),
            self.path_join(var_conf_dir, 'pmieconf'),
            self.path_join(var_conf_dir, 'pmlogrewrite')
        ])

        # Take PCP_LOG_DIR/pmlogger/`hostname` + PCP_LOG_DIR/pmmgr/`hostname`
        # The *default* directory structure for pmlogger is the following:
        # Dir: PCP_LOG_DIR/pmlogger/HOST/ (we only collect the HOST data
        # itself)
        # - YYYYMMDD.HH.MM.{N,N.index,N.meta} N in [0,1,...]
        # - Latest
        # - pmlogger.{log,log.prior}
        #
        # Can be changed via configuration in PCP_SYSCONF_DIR/pmlogger/control
        # As a default strategy, collect up to 100MB data from each dir.
        # Can be overwritten either via pcp.pcplogsize option or all_logs.
        self.pcp_hostname = gethostname()

        # Make sure we only add the two dirs if hostname is set, otherwise
        # we would collect everything
        if self.pcp_hostname != '':
            # collect pmmgr logs up to 'pmmgrlogs' size limit
            path = self.path_join(self.pcp_log_dir, 'pmmgr',
                                  self.pcp_hostname, '*')
            self.add_copy_spec(path, sizelimit=self.sizelimit, tailit=False)
            # collect newest pmlogger logs up to 'pmloggerfiles' count
            files_collected = 0
            path = self.path_join(self.pcp_log_dir, 'pmlogger',
                                  self.pcp_hostname, '*')
            pmlogger_ls = self.exec_cmd("ls -t1 %s" % path)
            if pmlogger_ls['status'] == 0:
                for line in pmlogger_ls['output'].splitlines():
                    self.add_copy_spec(line, sizelimit=0)
                    files_collected = files_collected + 1
                    if self.countlimit and files_collected == self.countlimit:
                        break

        self.add_copy_spec([
            # Collect PCP_LOG_DIR/pmcd and PCP_LOG_DIR/NOTICES
            self.path_join(self.pcp_log_dir, 'pmcd'),
            self.path_join(self.pcp_log_dir, 'NOTICES*'),
            # Collect PCP_VAR_DIR/pmns
            self.path_join(self.pcp_var_dir, 'pmns'),
            # Also collect any other log and config files
            # (as suggested by fche)
            self.path_join(self.pcp_log_dir, '*/*.log*'),
            self.path_join(self.pcp_log_dir, '*/*/*.log*'),
            self.path_join(self.pcp_log_dir, '*/*/config*')
        ])

        # Collect a summary for the current day
        res = self.collect_cmd_output('pcp')
        if res['status'] == 0:
            for line in res['output'].splitlines():
                if line.startswith(' pmlogger:'):
                    arc = line.split()[-1]
                    self.add_cmd_output(
                        "pmstat -S 00:00 -T 23:59 -t 5m -x -a %s" % arc,
                        root_symlink="pmstat"
                    )
                    break

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Red Hat, Inc., Lev Veyde <lveyde@redhat.com>
# Copyright (C) 2014 Red Hat, Inc., Sandro Bonazzola <sbonazzo@redhat.com>
# Copyright (C) 2014 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
# Copyright (C) 2010 Red Hat, Inc.

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import re
import signal


from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt
from sos.utilities import is_executable


# Class name must be the same as file name and method names must not change
class Ovirt(Plugin, RedHatPlugin):

    short_desc = 'oVirt Engine'

    plugin_name = "ovirt"
    profiles = ('virt',)

    packages = (
        'ovirt-engine',
        'ovirt-engine-dwh',
        'ovirt-engine-reports',
        'ovirt-engine-metrics',
        'ovirt-engine-setup',
        'ovirt-vmconsole',
        'ovirt-scheduler-proxy',
        'rhevm',
        'rhevm-dwh',
        'rhevm-reports'
    )

    DB_PASS_FILES = re.compile(
        flags=re.VERBOSE,
        pattern=r"""^/etc/
        (rhevm|ovirt-engine|ovirt-engine-dwh)/
        (engine.conf|ovirt-engine-dwhd.conf)
        (\.d/.+.conf.*?)?$"""
    )

    DEFAULT_SENSITIVE_KEYS = (
        'ENGINE_DB_PASSWORD:ENGINE_PKI_TRUST_STORE_PASSWORD:'
        'ENGINE_PKI_ENGINE_STORE_PASSWORD:DWH_DB_PASSWORD'
    )

    option_list = [
        PluginOpt('jbosstrace', default=True,
                  desc='Enable oVirt Engine JBoss stack trace collection'),
        PluginOpt('sensitive_keys', default=DEFAULT_SENSITIVE_KEYS,
                  desc='Sensitive keys to be masked in post-processing'),
        PluginOpt('heapdump', default=False,
                  desc='Collect heap dumps from /var/log/ovirt-engine/dump/')
    ]

    def setup(self):
        if self.get_option('jbosstrace') and self.is_installed('ovirt-engine'):
            engine_pattern = r"^ovirt-engine\ -server.*jboss-modules.jar"
            pgrep = "pgrep -f '%s'" % engine_pattern
            r = self.exec_cmd(pgrep)
            engine_pids = [int(x) for x in r['output'].splitlines()]
            if not engine_pids:
                self.soslog.error('Unable to get ovirt-engine pid')
                self.add_alert('Unable to get ovirt-engine pid')
            for pid in engine_pids:
                try:
                    # backtrace written to '/var/log/ovirt-engine/console.log
                    os.kill(pid, signal.SIGQUIT)
                except OSError as e:
                    self.soslog.error('Unable to send signal to %d' % pid, e)

        self.add_forbidden_path([
            '/etc/ovirt-engine/.pgpass',
            '/etc/rhevm/.pgpass'
        ])

        if not self.get_option('heapdump'):
            self.add_forbidden_path('/var/log/ovirt-engine/dump')
            self.add_cmd_output('ls -l /var/log/ovirt-engine/dump/')

        certificates = [
            '/etc/pki/ovirt-engine/ca.pem',
            '/etc/pki/ovirt-engine/apache-ca.pem',
            '/etc/pki/ovirt-engine/certs/engine.cer',
            '/etc/pki/ovirt-engine/certs/apache.cer',
            '/etc/pki/ovirt-engine/certs/websocket-proxy.cer',
            '/etc/pki/ovirt-engine/certs/jboss.cer',
            '/etc/pki/ovirt-engine/certs/imageio-proxy.cer',
            '/etc/pki/ovirt-engine/certs/ovirt-provider-ovn.cer',
        ]

        keystores = [
            ('mypass', '/etc/pki/ovirt-engine/.truststore'),
            ('changeit', '/var/lib/ovirt-engine/external_truststore'),
        ]

        self.add_cmd_output([
            # Copy all engine tunables and domain information
            "engine-config --all",
            # clearer diff from factory defaults (only on ovirt>=4.2.8)
            "engine-config -d",
        ])

        self.add_cmd_output([
            # process certificate files
            "openssl x509 -in %s -text -noout" % c for c in certificates
        ])

        self.add_cmd_output([
            # process TrustStore certificates
            "keytool -list -storepass %s -rfc -keystore %s" %
            (p, c) for (p, c) in keystores
        ])

        # 3.x line uses engine-manage-domains, 4.x uses ovirt-aaa-jdbc-tool
        manage_domains = 'engine-manage-domains'
        extensions_tool = 'ovirt-engine-extensions-tool'
        jdbc_tool = 'ovirt-aaa-jdbc-tool'

        if is_executable(manage_domains):
            self.add_cmd_output('%s list' % manage_domains)
        if is_executable(extensions_tool):
            self.add_cmd_output('%s info list-extensions' % extensions_tool)
        if is_executable('ovirt-aaa-jdbc-tool'):
            subcmds = [
                'query --what=user',
                'query --what=group',
                'settings show'
            ]

            self.add_cmd_output(['%s %s' % (jdbc_tool, sc) for sc in subcmds])

        # Copy engine config files.
        self.add_copy_spec([
            "/etc/ovirt-engine",
            "/etc/rhevm/",
            "/etc/ovirt-engine-dwh",
            "/etc/ovirt-engine-reports",
            "/etc/ovirt-engine-metrics",
            "/etc/ovirt-engine-setup",
            "/etc/ovirt-vmconsole",
            "/var/log/ovirt-engine",
            "/var/log/ovirt-engine-dwh",
            "/var/log/ovirt-engine-reports",
            "/var/log/ovirt-scheduler-proxy",
            "/var/log/rhevm",
            "/etc/sysconfig/ovirt-engine",
            "/usr/share/ovirt-engine/conf",
            "/var/log/ovirt-guest-agent",
            "/var/lib/ovirt-engine/setup-history.txt",
            "/var/lib/ovirt-engine/setup/answers",
            "/var/lib/ovirt-engine/external_truststore",
            "/var/tmp/ovirt-engine/config",
            "/var/lib/ovirt-engine/jboss_runtime/config",
            "/var/lib/ovirt-engine-reports/jboss_runtime/config"
        ])

        # Copying host certs; extra copy the hidden .truststore file
        self.add_forbidden_path([
            "/etc/pki/ovirt-engine/keys",
            "/etc/pki/ovirt-engine/private"
        ])
        self.add_copy_spec([
            "/etc/pki/ovirt-engine/",
            "/etc/pki/ovirt-engine/.truststore",
        ])

    def postproc(self):
        """
        Obfuscate sensitive keys.
        """
        self.do_file_sub(
            "/etc/ovirt-engine/engine-config/engine-config.properties",
            r"Password.type=(.*)",
            r"Password.type=********"
        )
        self.do_file_sub(
            "/etc/rhevm/rhevm-config/rhevm-config.properties",
            r"Password.type=(.*)",
            r"Password.type=********"
        )

        engine_files = (
            'ovirt-engine.xml',
            'ovirt-engine_history/current/ovirt-engine.v1.xml',
            'ovirt-engine_history/ovirt-engine.boot.xml',
            'ovirt-engine_history/ovirt-engine.initial.xml',
            'ovirt-engine_history/ovirt-engine.last.xml',
        )
        for filename in engine_files:
            self.do_file_sub(
                "/var/tmp/ovirt-engine/config/%s" % filename,
                r"<password>(.*)</password>",
                r"<password>********</password>"
            )

        self.do_file_sub(
            "/etc/ovirt-engine/redhatsupportplugin.conf",
            r"proxyPassword=(.*)",
            r"proxyPassword=********"
        )

        passwd_files = [
            "logcollector.conf",
            "imageuploader.conf",
            "isouploader.conf"
        ]
        for conf_file in passwd_files:
            conf_path = self.path_join("/etc/ovirt-engine", conf_file)
            self.do_file_sub(
                conf_path,
                r"passwd=(.*)",
                r"passwd=********"
            )
            self.do_file_sub(
                conf_path,
                r"pg-pass=(.*)",
                r"pg-pass=********"
            )

        sensitive_keys = self.DEFAULT_SENSITIVE_KEYS
        # Handle --alloptions case which set this to True.
        keys_opt = self.get_option('sensitive_keys')
        if keys_opt and keys_opt is not True:
            sensitive_keys = keys_opt
        key_list = [x for x in sensitive_keys.split(':') if x]
        for key in key_list:
            self.do_path_regex_sub(
                self.DB_PASS_FILES,
                r'{key}=(.*)'.format(key=key),
                r'{key}=********'.format(key=key)
            )

        # Answer files contain passwords.
        # Replace all keys that have 'password' in them, instead of hard-coding
        # here the list of keys, which changes between versions.
        # Sadly, the engine admin password prompt name does not contain
        # 'password'... so neither does the env key.
        for item in (
            'password',
            'OVESETUP_CONFIG_ADMIN_SETUP',
        ):
            self.do_path_regex_sub(
                r'/var/lib/ovirt-engine/setup/answers/.*',
                re.compile(
                    r'(?P<key>[^=]*{item}[^=]*)=.*'.format(item=item),
                    flags=re.IGNORECASE
                ),
                r'\g<key>=********'
            )

        # aaa profiles contain passwords
        protect_keys = [
            "vars.password",
            "pool.default.auth.simple.password",
            "pool.default.ssl.truststore.password",
            "config.datasource.dbpassword"
        ]
        regexp = r"((?m)^\s*#*(%s)\s*=\s*)(.*)" % "|".join(protect_keys)

        self.do_path_regex_sub(r"/etc/ovirt-engine/aaa/.*\.properties", regexp,
                               r"\1*********")

# vim: expandtab tabstop=4 shiftwidth=4



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.parsers import SoSCleanerParser
from sos.cleaner.mappings.ip_map import SoSIPMap


class SoSIPParser(SoSCleanerParser):
    """Handles parsing for IP addresses"""

    name = 'IP Parser'
    regex_patterns = [
        # IPv4 with or without CIDR
        r'((?<!(-|\.|\d))([0-9]{1,3}\.){3}([0-9]){1,3}(\/([0-9]{1,2}))?)'
    ]
    skip_line_patterns = [
        # don't match package versions recorded in journals
        r'.*dnf\[.*\]:'
    ]

    skip_files = [
        # skip these as version numbers will frequently look like IP addresses
        # when using regex matching
        'installed-debs',
        'installed-rpms',
        'sos_commands/dpkg',
        'sos_commands/python/pip_list',
        'sos_commands/rpm',
        'sos_commands/yum/.*list.*',
        'sos_commands/snappy/snap_list_--all',
        'sos_commands/vulkan/vulkaninfo',
        'var/log/.*dnf.*',
        'var/log/.*packag.*',  # get 'packages' and 'packaging' logs
        '.*(version|release)(\\.txt)?$',  # obvious version files
    ]

    map_file_key = 'ip_map'
    compile_regexes = False

    def __init__(self, config):
        self.mapping = SoSIPMap()
        super(SoSIPParser, self).__init__(config)



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt
import re


class Dlm(Plugin, IndependentPlugin):

    short_desc = 'DLM (Distributed lock manager)'

    plugin_name = "dlm"
    profiles = ("cluster", )
    packages = ("cman", "dlm", "pacemaker")
    option_list = [
        PluginOpt('lockdump', default=False, desc='capture lock dumps for DLM')
    ]

    def setup(self):
        self.add_copy_spec([
            "/etc/sysconfig/dlm"
        ])
        self.add_cmd_output([
            "dlm_tool log_plock",
            "dlm_tool dump",
            "dlm_tool ls -n"
        ])
        if self.get_option("lockdump"):
            self.do_lockdump()

    def do_lockdump(self):
        dlm_tool = "dlm_tool ls"
        result = self.collect_cmd_output(dlm_tool)
        if result["status"] != 0:
            return

        lock_exp = r'^name\s+([^\s]+)$'
        lock_re = re.compile(lock_exp, re.MULTILINE)
        for lockspace in lock_re.findall(result["output"]):
            self.add_cmd_output(
                "dlm_tool lockdebug -svw '%s'" % lockspace,
                suggest_filename="dlm_locks_%s" % lockspace
            )

# vim: et ts=4 sw=4



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Buildah(Plugin, RedHatPlugin):

    short_desc = 'Buildah container and image builder'

    plugin_name = 'buildah'
    packages = ('buildah',)
    profiles = ('container',)

    def setup(self):
        subcmds = [
            'containers',
            'containers --all',
            'images',
            'images --all',
            'version'
        ]

        self.add_cmd_output(["buildah %s" % sub for sub in subcmds])

        def make_chowdah(aurdah):
            chowdah = self.exec_cmd(aurdah)
            chowdah['auutput'] = chowdah.pop('output')
            chowdah['is_wicked_pissah'] = chowdah.pop('status') == 0
            return chowdah

        containahs = make_chowdah('buildah containers -n')
        if containahs['is_wicked_pissah']:
            for containah in containahs['auutput'].splitlines():
                # obligatory Tom Brady
                goat = containah.split()[-1]
                self.add_cmd_output('buildah inspect -t container %s' % goat,
                                    subdir='containers')

        pitchez = make_chowdah('buildah images -n')
        if pitchez['is_wicked_pissah']:
            for pitchah in pitchez['auutput'].splitlines():
                brady = pitchah.split()[1]
                self.add_cmd_output('buildah inspect -t image %s' % brady,
                                    subdir='images')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Ntp(Plugin):

    short_desc = 'Network Time Protocol'
    plugin_name = "ntp"
    profiles = ('system', 'services')

    packages = ('ntp',)

    def setup(self):

        self.add_copy_spec("/etc/ntp.conf", tags="ntp_conf")

        self.add_copy_spec([
            "/etc/ntp/step-tickers",
            "/etc/ntp/ntpservers"
        ])
        self.add_cmd_output([
            "ntptime",
            "ntpq -pn"
        ], cmd_as_tag=True)

        ids = self.collect_cmd_output('ntpq -c as')
        if ids['status'] == 0:
            for asid in [i.split()[1] for i in ids['output'].splitlines()[3:]]:
                self.add_cmd_output("ntpq -c 'rv %s'" % asid)


class RedHatNtp(Ntp, RedHatPlugin):

    def setup(self):
        super(RedHatNtp, self).setup()
        self.add_copy_spec("/etc/sysconfig/ntpd")
        self.add_cmd_output("ntpstat")


class DebianNtp(Ntp, DebianPlugin, UbuntuPlugin):

    def setup(self):
        super(DebianNtp, self).setup()
        self.add_copy_spec('/etc/default/ntp')


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class DefaultEnv(Plugin, IndependentPlugin):

    plugin_name = 'default_env_test'
    short_desc = 'Fake plugin to test default env var handling'

    def setup(self):
        self.set_default_cmd_environment({
            'TORVALDS': 'Linus',
            'GREATESTSPORT': 'hockey'
        })

        self.add_cmd_output(
            "sh -c 'echo Does '$TORVALDS' play '$GREATESTSPORT'?'",
            suggest_filename='env_var_test'
        )

        self.add_env_var(['TORVALDS', 'GREATESTSPORT'])



# Copyright (C) 2013 Adam Stokes <adam.stokes@ubuntu.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


class Juju(Plugin, UbuntuPlugin):

    short_desc = 'Juju orchestration tool'

    plugin_name = 'juju'
    profiles = ('virt', 'sysmgmt')

    # Using files instead of packages here because there is no identifying
    # package on a juju machine.
    files = ('/var/log/juju',)

    def setup(self):
        # Juju service names are not consistent through deployments,
        # so we need to use a wildcard to get the correct service names.
        for service in self.get_service_names("juju*"):
            self.add_journal(service)
            self.add_service_status(service)

        # Get agent configs for each agent.
        self.add_copy_spec("/var/lib/juju/agents/*/agent.conf")

        # Get a directory listing of /var/log/juju and /var/lib/juju
        self.add_cmd_output([
            "ls -alRh /var/log/juju*",
            "ls -alRh /var/lib/juju*"
        ])

        if self.get_option("all_logs"):
            # /var/lib/juju used to be in the default capture moving here
            # because it usually was way to big.  However, in most cases you
            # want all logs you want this too.
            self.add_copy_spec([
                "/var/log/juju",
                "/var/lib/juju"
            ])
            self.add_forbidden_path("/var/lib/juju/kvm")
        else:
            # We need this because we want to collect to the limit of all
            # logs in the directory.
            self.add_copy_spec("/var/log/juju/*.log")

    def postproc(self):
        agents_path = "/var/lib/juju/agents/*"
        protect_keys = [
            "sharedsecret",
            "apipassword",
            "oldpassword",
            "statepassword",
        ]

        # Redact simple yaml style "key: value".
        keys_regex = r"((?m)^\s*(%s)\s*:\s*)(.*)" % "|".join(protect_keys)
        sub_regex = r"\1*********"
        self.do_path_regex_sub(agents_path, keys_regex, sub_regex)
        # Redact certificates
        self.do_file_private_sub(agents_path)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class LogRotate(Plugin, IndependentPlugin):

    short_desc = 'LogRotate service'

    plugin_name = 'logrotate'
    profiles = ('system',)

    var_puppet_gen = "/var/lib/config-data/puppet-generated/crond"

    def setup(self):
        self.add_cmd_output("logrotate --debug /etc/logrotate.conf",
                            suggest_filename="logrotate_debug")
        self.add_copy_spec([
            "/etc/logrotate*",
            "/var/lib/logrotate.status",
            "/var/lib/logrotate/logrotate.status",
            self.var_puppet_gen + "/etc/logrotate-crond.conf",
            self.var_puppet_gen + "/var/spool/cron/root"
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Usb(Plugin, IndependentPlugin):

    short_desc = 'USB devices'

    plugin_name = "usb"
    profiles = ('hardware',)

    def setup(self):
        self.add_copy_spec("/sys/bus/usb")

        self.add_cmd_output([
            "lsusb",
            "lsusb -v",
            "lsusb -t"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2020 Red Hat, Inc., Cedric Jeanneret <cjeanner@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


# Notes:
# - The Validation Framework is, for now, linked to openstack and tripleo
# - Since the intent is to open it to other product (we can validate anything)
#   this plugin has a generic name.
# - The Framework is targeted at Red Hat products, at least for now.
class ValidationFramework(Plugin, RedHatPlugin):

    short_desc = 'Logs provided by the Validation Framework'

    plugin_name = 'validation_framework'
    profiles = ('openstack', 'openstack_controller', 'openstack_compute')
    packages = ('tripleo-validations',)

    def setup(self):
        self.add_copy_spec('/var/log/validations/')

    def postproc(self):
        # Use a generic match in order to clean things up.
        # It is not expected to get any secrets in here, but we'd better
        # ensure it's clean.
        secrets = r'(".*(key|password|pass|secret|database_connection))' \
                  r'([":\s]+)(.*[^"])([",]+)'

        self.do_path_regex_sub('/var/log/validations/',
                               secrets, r'\1\3*********\5')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Ntb(Plugin, RedHatPlugin):

    short_desc = 'Linux PCI-Express Non-Transparent Bridge'
    plugin_name = 'ntb'
    profiles = ('hardware', )

    def setup(self):
        # NTB is hardwired at PCI Bus 0, device 3, function 0 on Intel
        # processors (see page 8 in
        # http://download.intel.com/design/intarch/papers/323328.pdf).
        self.add_copy_spec([
            '/sys/kernel/debug/ntb_hw_intel/0000:*/info',
            '/sys/kernel/debug/ntb_transport/0000:*/qp*/stats'
        ])


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Veritas(Plugin, RedHatPlugin):

    short_desc = 'Veritas software'

    plugin_name = 'veritas'
    profiles = ('cluster', 'storage')

    # Information about VRTSexplorer obtained from
    # http://seer.entsupport.symantec.com/docs/243150.htm
    option_list = [
        PluginOpt('script', default='/opt/VRTSspt/VRTSexplorer',
                  desc='Path to VRTSexploer script')
    ]

    def check_enabled(self):
        return self.path_isfile(self.get_option("script"))

    def setup(self):
        """ interface with vrtsexplorer to capture veritas related data """
        r = self.exec_cmd(self.get_option("script"))
        if r['status'] == 0:
            tarfile = ""
            for line in r['output']:
                line = line.strip()
                tarfile = self.do_regex_find_all(r"ftp (.*tar.gz)", line)
            if len(tarfile) == 1:
                self.add_copy_spec(tarfile[0])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Gdm(Plugin, IndependentPlugin):

    short_desc = 'GNOME display manager'

    plugin_name = 'gdm'
    profiles = ('desktop',)
    packages = ('gdm', 'gdm3',)
    services = ('gdm',)

    def setup(self):
        self.add_copy_spec([
            "/etc/gdm/*",
            "/etc/gdm3/*"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re


from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class OpenStackDatabase(Plugin):

    short_desc = 'Openstack Database Information'
    plugin_name = 'openstack_database'
    profiles = ('openstack', 'openstack_controller')

    option_list = [
        PluginOpt('dump', default=False, desc='Dump select databases'),
        PluginOpt('dumpall', default=False, desc='Dump ALL databases')
    ]

    databases = [
        'cinder',
        'glance',
        'heat',
        'ironic',
        'keystone',
        'mistral',
        '(.*)?neutron',
        'nova.*'
    ]

    def setup(self):
        # determine if we're running databases on the host or in a container
        _db_containers = [
            'galera-bundle-.*',  # overcloud
            'mysql'  # undercloud
        ]

        cname = None
        for container in _db_containers:
            cname = self.get_container_by_name(container)
            if cname:
                break

        fname = "clustercheck_%s" % cname if cname else None
        self.add_cmd_output('clustercheck', container=cname, timeout=15,
                            suggest_filename=fname)

        if self.get_option('dump') or self.get_option('dumpall'):
            db_dump = self.get_mysql_db_string(container=cname)
            db_cmd = "mysqldump --opt %s" % db_dump

            self.add_cmd_output(db_cmd, suggest_filename='mysql_dump.sql',
                                sizelimit=0, container=cname)

    def get_mysql_db_string(self, container=None):

        if self.get_option('dumpall'):
            return '--all-databases'

        collect = []
        dbs = self.exec_cmd('mysql -e "show databases;"', container=container)

        for db in dbs['output'].splitlines():
            if any([re.match(db, reg) for reg in self.databases]):
                collect.append(db)

        return '-B ' + ' '.join(d for d in collect)


class RedHatOpenStackDatabase(OpenStackDatabase, RedHatPlugin):

    packages = ('openstack-selinux', )


# vim: set et ts=4 sw=4 :



# Copyright (C) 2007-2012 Red Hat, Inc., Ben Turner <bturner@redhat.com>
# Copyright (C) 2012 Adam Stokes <adam.stokes@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class IscsiTarget(Plugin):

    short_desc = 'iSCSI target'

    plugin_name = "iscsitarget"
    profiles = ('storage',)


class RedHatIscsiTarget(IscsiTarget, RedHatPlugin):

    packages = ('scsi-target-utils',)

    def setup(self):
        super(RedHatIscsiTarget, self).setup()
        self.add_copy_spec("/etc/tgt/targets.conf")
        self.add_cmd_output("tgtadm --lld iscsi --op show --mode target")


class DebianIscsiTarget(IscsiTarget, DebianPlugin, UbuntuPlugin):

    packages = ('iscsitarget',)

    def setup(self):
        super(DebianIscsiTarget, self).setup()
        self.add_copy_spec([
            "/etc/iet",
            "/etc/sysctl.d/30-iscsitarget.conf",
            "/etc/default/iscsitarget"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Fibrechannel(Plugin, RedHatPlugin):

    short_desc = 'Collect information on fibrechannel devices'

    plugin_name = 'fibrechannel'
    profiles = ('hardware', 'storage', 'system')
    files = ('/sys/class/fc_host', '/sys/class/fc_remote_ports')
    option_list = [
        PluginOpt('debug', default=True, desc='collect debugging logs')
    ]

    # vendor specific debug paths
    debug_paths = [
        '/sys/kernel/debug/qla2*/',
        '/sys/kernel/debug/fnic/',
    ]

    def setup(self):
        self.add_device_cmd("udevadm info -a %(dev)s", devices='fibre')
        if self.get_option('debug'):
            self.add_copy_spec(self.debug_paths)

        self.add_cmd_output([
            "hbacmd listhbas",
            "hbacmd ServerAttributes"
        ])

        # collect Hbaattributes and Portattributes of WWN
        listhbas = self.collect_cmd_output("hbacmd listhbas")
        if listhbas['status'] == 0:
            for line in listhbas['output'].splitlines():
                if 'Port WWN' in line:
                    dev = line.split()[3]
                    self.add_cmd_output([
                        "hbacmd HbaAttributes %s" % dev,
                        "hbacmd PortAttributes %s" % dev,
                        "hbacmd GetXcvrData %s" % dev
                    ])
# vim: set et ts=4 sw=4 :



#!/usr/bin/env python

from setuptools import setup, find_packages
from sos import __version__ as VERSION


setup(
    name='sos',
    version=VERSION,
    install_requires=['pexpect', 'pyyaml'],
    description=(
        'A set of tools to gather troubleshooting information from a system'
    ),
    author='Bryn M. Reeves',
    author_email='bmr@redhat.com',
    maintainer='Jake Hunsaker',
    maintainer_email='jhunsake@redhat.com',
    url='https://github.com/sosreport/sos',
    license="GPLv2+",
    scripts=['bin/sos', 'bin/sosreport', 'bin/sos-collector'],
    data_files=[
        ('share/man/man1', ['man/en/sosreport.1', 'man/en/sos-report.1',
                            'man/en/sos.1', 'man/en/sos-collect.1',
                            'man/en/sos-collector.1', 'man/en/sos-clean.1',
                            'man/en/sos-mask.1', 'man/en/sos-help.1']),
        ('share/man/man5', ['man/en/sos.conf.5']),
        ('share/licenses/sos', ['LICENSE']),
        ('share/doc/sos', ['AUTHORS', 'README.md']),
        ('config', ['sos.conf'])
    ],
    packages=find_packages(include=['sos', 'sos.*'])
)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Red Hat, Inc., Pavel Moravec <pmoravec@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin)


class ForemanProxy(Plugin):

    short_desc = 'Foreman Smart Proxy systems management'

    plugin_name = 'foreman_proxy'
    profiles = ('sysmgmt',)
    packages = ('foreman-proxy',)

    def setup(self):
        self.add_file_tags({
            '/var/log/foreman-proxy/proxy.log.*': 'foreman_proxy_log',
            '/etc/foreman-proxy/settings.yml': 'foreman_proxy_conf'
        })

        self.add_forbidden_path([
            "/etc/foreman-proxy/*key.pem"
        ])

        self.add_copy_spec([
            "/etc/foreman-proxy/",
            "/etc/smart_proxy_dynflow_core/settings.yml",
            "/var/log/foreman-proxy/*log*",
            "/var/log/{}*/katello-reverse-proxy_access_ssl.log*".format(
                self.apachepkg),
            "/var/log/{}*/katello-reverse-proxy_error_ssl.log*".format(
                self.apachepkg),
            "/var/log/{}*/rhsm-pulpcore-https-*_access_ssl.log*".format(
                self.apachepkg),
            "/var/log/{}*/rhsm-pulpcore-https-*_error_ssl.log*".format(
                self.apachepkg),
        ])

        # collect http[|s]_proxy env.variables
        self.add_env_var(["http_proxy", "https_proxy"])

    def postproc(self):
        self.do_path_regex_sub(
            r"/etc/foreman-proxy/(.*)((conf)(.*)?)",
            r"((\:|\s*)(passw|cred|token|secret|key).*(\:\s|=))(.*)",
            r"\1********")
        # yaml values should be alphanumeric
        self.do_path_regex_sub(
            r"/etc/foreman-proxy/(.*)((yaml|yml)(.*)?)",
            r"((\:|\s*)(passw|cred|token|secret|key).*(\:\s|=))(.*)",
            r'\1"********"')


# Child classes needed to declare the apachepkg attr properly per distro

class RedHatForemanProxy(ForemanProxy, RedHatPlugin):

    apachepkg = 'httpd'


class DebianForemanProxy(ForemanProxy, DebianPlugin, UbuntuPlugin):

    apachepkg = 'apache2'


# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.policies.package_managers import PackageManager


class RpmPackageManager(PackageManager):
    """Package Manager for RPM-based distributions
    """

    query_command = 'rpm -qa --queryformat "%{NAME}|%{VERSION}|%{RELEASE}\\n"'
    query_path_command = 'rpm -qf'
    files_command = 'rpm -qal'
    verify_command = 'rpm -V'
    verify_filter = ["debuginfo", "-devel"]


# vim: set et ts=4 sw=4 :



# Copyright (C) 2013 Red Hat, Inc., Brent Eagles <beagles@redhat.com>
# Copyright (C) 2017 Red Hat, Inc., Martin Schuppert <mschuppert@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin
import os


class OpenStackNeutron(Plugin):

    short_desc = 'OpenStack Networking'
    plugin_name = "openstack_neutron"
    profiles = ('openstack', 'openstack_controller', 'openstack_compute')

    var_puppet_gen = "/var/lib/config-data/puppet-generated/neutron"

    def setup(self):
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/neutron/",
            ])
        else:
            self.add_copy_spec([
                "/var/log/neutron/*.log",
            ])

        self.add_copy_spec([
            "/etc/neutron/",
            self.var_puppet_gen + "/etc/neutron/",
            self.var_puppet_gen + "/etc/default/neutron-server",
            self.var_puppet_gen + "/etc/my.cnf.d/tripleo.cnf"
        ])
        # copy whole /var/lib/neutron except for potentially huge lock subdir;
        # rather take a list of files in the dir only
        self.add_copy_spec("/var/lib/neutron/")
        self.add_forbidden_path("/var/lib/neutron/lock")
        self.add_cmd_output("ls -laZR /var/lib/neutron/lock")

        vars_all = [p in os.environ for p in [
                    'OS_USERNAME', 'OS_PASSWORD']]

        vars_any = [p in os.environ for p in [
                    'OS_TENANT_NAME', 'OS_PROJECT_NAME']]

        if not (all(vars_all) and any(vars_any)):
            self.soslog.warning("Not all environment variables set. Source "
                                "the environment file for the user intended "
                                "to connect to the OpenStack environment.")
        else:
            self.add_cmd_output("openstack subnet list")
            self.add_cmd_output("openstack port list")
            self.add_cmd_output("openstack router list")
            self.add_cmd_output("openstack network agent list")
            self.add_cmd_output("openstack network list")
            self.add_cmd_output("openstack extension list")
            self.add_cmd_output("openstack floating ip list")
            self.add_cmd_output("openstack security group list")

    def apply_regex_sub(self, regexp, subst):
        self.do_path_regex_sub("/etc/neutron/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/neutron/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "rabbit_password", "qpid_password", "nova_admin_password",
            "xenapi_connection_password", "password", "server_auth",
            "admin_password", "metadata_proxy_shared_secret", "eapi_password",
            "crd_password", "primary_l3_host_password", "serverauth",
            "ucsm_password", "ha_vrrp_auth_password", "ssl_key_password",
            "nsx_password", "vcenter_password", "edge_appliance_password",
            "tenant_admin_password", "apic_password", "transport_url",
            "memcache_secret_key"
        ]
        connection_keys = ["connection"]

        self.apply_regex_sub(
            r"((?m)^\s*(%s)\s*=\s*)(.*)" % "|".join(protect_keys),
            r"\1*********"
        )
        self.apply_regex_sub(
            r"((?m)^\s*(%s)\s*=\s*(.*)://(\w*):)(.*)(@(.*))" %
            "|".join(connection_keys),
            r"\1*********\6"
        )


class DebianNeutron(OpenStackNeutron, DebianPlugin, UbuntuPlugin):
    packages = (
        'neutron-common',
        'neutron-plugin-cisco',
        'neutron-plugin-linuxbridge-agent',
        'neutron-plugin-nicira',
        'neutron-plugin-openvswitch',
        'neutron-plugin-openvswitch-agent',
        'neutron-plugin-ryu',
        'neutron-plugin-ryu-agent',
        'neutron-server',
        'python-neutron',
        'python-neutronclient'
    )

    def check_enabled(self):
        return self.is_installed("neutron-common")

    def setup(self):
        super(DebianNeutron, self).setup()
        self.add_copy_spec("/etc/sudoers.d/neutron_sudoers")


class RedHatNeutron(OpenStackNeutron, RedHatPlugin):

    packages = ('openstack-selinux',)

    def setup(self):
        super(RedHatNeutron, self).setup()
        self.add_copy_spec("/etc/sudoers.d/neutron-rootwrap")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Red Hat, Inc., Pierre Carrier <pcarrier@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, SoSPredicate)
from glob import glob


class Sssd(Plugin):

    short_desc = 'System security service daemon'

    plugin_name = "sssd"
    profiles = ('services', 'security', 'identity')
    packages = ('sssd', 'sssd-common')

    def setup(self):
        self.add_copy_spec([
            # main config file
            "/etc/sssd/sssd.conf",
            # SSSD 1.14
            "/etc/sssd/conf.d/*.conf",
            # dynamic Kerberos configuration
            "/var/lib/sss/pubconf/krb5.include.d/*"
        ])

        # add individual log files
        self.add_copy_spec(glob("/var/log/sssd/*log*"), tags='sssd_logs')

        # add memory cache
        self.add_copy_spec([
            "/var/lib/sss/mc/passwd",
            "/var/lib/sss/mc/group",
            "/var/lib/sss/mc/initgroups"
        ])

        # call sssctl commands only when sssd service is running,
        # otherwise the command timeouts
        sssd_pred = SoSPredicate(self, services=["sssd"])
        self.add_cmd_output("sssctl config-check", pred=sssd_pred)

        # if predicate fails, domain["status"] = None and thus we skip parsing
        # missing output
        domain = self.collect_cmd_output("sssctl domain-list", pred=sssd_pred)
        if domain['status'] == 0:
            for domain_name in domain['output'].splitlines():
                self.add_cmd_output("sssctl domain-status -o " + domain_name)

    def postproc(self):
        regexp = r"(\s*ldap_default_authtok\s*=\s*)\S+"

        self.do_file_sub("/etc/sssd/sssd.conf", regexp, r"\1********")
        self.do_path_regex_sub("/etc/sssd/conf.d/*", regexp, r"\1********")


class RedHatSssd(Sssd, RedHatPlugin):

    def setup(self):
        super(RedHatSssd, self).setup()


class DebianSssd(Sssd, DebianPlugin, UbuntuPlugin):

    def setup(self):
        super(DebianSssd, self).setup()
        self.add_copy_spec("/etc/default/sssd")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2009 Red Hat, Inc., Joey Boggs <jboggs@redhat.com>
# Copyright (C) 2012 Rackspace US, Inc.,
#                    Justin Shepherd <jshepher@rackspace.com>
# Copyright (C) 2013 Red Hat, Inc., Jeremy Agee <jagee@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackHorizon(Plugin):

    short_desc = 'OpenStack Horizon'

    plugin_name = "openstack_horizon"
    profiles = ('openstack', 'openstack_controller')
    var_puppet_gen = "/var/lib/config-data/puppet-generated"

    def setup(self):
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/horizon/",
            ])
        else:
            self.add_copy_spec([
                "/var/log/horizon/*.log",
            ])

        self.add_copy_spec([
            "/etc/openstack-dashboard/",
            self.var_puppet_gen + "/horizon/etc/openstack-dashboard/",
            self.var_puppet_gen + "/horizon/etc/httpd/conf/",
            self.var_puppet_gen + "/horizon/etc/httpd/conf.d/",
            self.var_puppet_gen + "/horizon/etc/httpd/conf.modules.d/*.conf",
            self.var_puppet_gen + "/memcached/etc/sysconfig/memcached"
        ])
        self.add_forbidden_path(
            "/etc/openstack-dashboard/local_settings.d/*.py[co]"
        )

    def postproc(self):
        var_puppet_gen = self.var_puppet_gen + "/horizon"
        protect_keys = [
            "SECRET_KEY", "EMAIL_HOST_PASSWORD"
        ]

        regexp = r"((?m)^\s*(%s)\s*=\s*)(.*)" % "|".join(protect_keys)
        self.do_path_regex_sub(
            r"/etc/openstack-dashboard/.*\.json",
            regexp, r"\1*********"
        )
        self.do_path_regex_sub(
            var_puppet_gen + r"/etc/openstack-dashboard/.*\.json",
            regexp, r"\1*********"
        )
        self.do_path_regex_sub(
            "/etc/openstack-dashboard/local_settings$",
            regexp, r"\1*********"
        )
        self.do_path_regex_sub(
            var_puppet_gen + "/etc/openstack-dashboard/local_settings$",
            regexp, r"\1*********"
        )


class DebianHorizon(OpenStackHorizon, DebianPlugin):

    packages = (
        'python-django-horizon',
        'openstack-dashboard',
        'openstack-dashboard-apache'
    )

    def setup(self):
        super(DebianHorizon, self).setup()
        self.add_copy_spec("/etc/apache2/sites-available/")


class UbuntuHorizon(OpenStackHorizon, UbuntuPlugin):

    packages = (
        'python-django-horizon',
        'openstack-dashboard',
        'openstack-dashboard-ubuntu-theme'
    )

    def setup(self):
        super(UbuntuHorizon, self).setup()
        self.add_copy_spec("/etc/apache2/conf.d/openstack-dashboard.conf")


class RedHatHorizon(OpenStackHorizon, RedHatPlugin):

    packages = ('openstack-selinux',)

    def setup(self):
        super(RedHatHorizon, self).setup()
        self.add_copy_spec("/etc/httpd/conf.d/openstack-dashboard.conf")
        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/httpd/horizon*")
        else:
            self.add_copy_spec([
                "/var/log/httpd/horizon*.log"
                "/var/log/httpd/"
            ])

# vim: set et ts=4 sw=4 :



# Copyright 2010 Red Hat, Inc.
# Author: Adam Stokes <astokes@fedoraproject.org>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


"""
This module houses the i18n setup and message function. The default is to use
gettext to internationalize messages.
"""
__version__ = "4.5.0"

import os
import sys

from argparse import ArgumentParser
from sos.options import SosListOption

import gettext
gettext_dir = "/usr/share/locale"
gettext_app = "sos"
gettext.bindtextdomain(gettext_app, gettext_dir)


def _default(msg):
    return gettext.dgettext(gettext_app, msg)


_sos = _default

# py3 < 3.6 compat
try:
    ModuleNotFoundError
except NameError:
    ModuleNotFoundError = ImportError


class SoS():
    """Main entrypoint for sos from the command line

    Upon intialization, this class loads the basic option parser which will
    include the options shared by support components/subcommands. This is also
    where all subcommands present in the local installation are discovered,
    loaded, and if a matching one is found, intialized.
    """

    def __init__(self, args):
        self.cmdline = args
        # define the local subcommands that exist on the system
        # first import the necessary module, then add an entry to the dict that
        # follows the tuple format (class, [aliases]), where aliases is a list
        # of shorthand names to accept in place of the full subcommand
        # if no aliases are desired, pass an empty list
        import sos.report
        import sos.cleaner
        import sos.help
        self._components = {
            'report': (sos.report.SoSReport, ['rep']),
            'clean': (sos.cleaner.SoSCleaner, ['cleaner', 'mask']),
            'help': (sos.help.SoSHelper, [])
        }
        # some distros do not want pexpect as a default dep, so try to load
        # collector here, and if it fails add an entry that implies it is at
        # least present on this installation
        try:
            import sos.collector
            self._components['collect'] = (sos.collector.SoSCollector,
                                           ['collector'])
        except ModuleNotFoundError as err:
            import sos.missing
            if 'sos.collector' in err.msg:
                # is not locally installed - packaged separately
                self._components['collect'] = (sos.missing.MissingCollect, [])
            elif 'pexpect' in err.msg:
                # cannot be imported due to missing the pexpect dep
                self._components['collect'] = (sos.missing.MissingPexpect, [])
            else:
                # we failed elsewhere, re-raise the exception
                raise
        # build the top-level parser
        _com_string = ''
        for com in self._components:
            aliases = self._components[com][1]
            aliases.insert(0, com)
            _com = ', '.join(aliases)
            desc = self._components[com][0].desc
            _com_string += (
                "\t{com:<30}{desc}\n".format(com=_com, desc=desc)
            )
        usage_string = ("%(prog)s <component> [options]\n\n"
                        "Available components:\n")
        usage_string = usage_string + _com_string
        epilog = ("See `sos <component> --help` for more information")
        self.parser = ArgumentParser(usage=usage_string, epilog=epilog)
        self.parser.register('action', 'extend', SosListOption)
        # set the component subparsers
        self.subparsers = self.parser.add_subparsers(
            dest='component',
            metavar='component',
            help='sos component to run'
        )
        self.subparsers.required = True
        # now build the parser for each component.
        # this needs to be done here, as otherwise --help will be unavailable
        # for the component subparsers
        for comp in self._components:
            _com_subparser = self.subparsers.add_parser(
                comp,
                aliases=self._components[comp][1],
                prog="sos %s" % comp
            )
            _com_subparser.usage = "sos %s [options]" % comp
            _com_subparser.register('action', 'extend', SosListOption)
            self._add_common_options(_com_subparser)
            self._components[comp][0].add_parser_options(parser=_com_subparser)
            _com_subparser.set_defaults(component=comp)
        self.args = self.parser.parse_args(self.cmdline)
        self._init_component()

    def _add_common_options(self, parser):
        """Adds the options shared across components to the parser
        """
        global_grp = parser.add_argument_group('Global Options')
        global_grp.add_argument("--batch", default=False, action="store_true",
                                help="Do not prompt interactively")
        global_grp.add_argument("--config-file", type=str, action="store",
                                dest="config_file",
                                default="/etc/sos/sos.conf",
                                help="specify alternate configuration file")
        global_grp.add_argument("--debug", action="store_true", dest="debug",
                                help="enable interactive debugging using the "
                                     "python debugger")
        global_grp.add_argument("-q", "--quiet", action="store_true",
                                dest="quiet", default=False,
                                help="only print fatal errors")
        global_grp.add_argument("-s", "--sysroot", action="store",
                                dest="sysroot", default=None,
                                help="system rootdir path (default='/')")
        global_grp.add_argument("--tmp-dir", action="store", dest="tmp_dir",
                                default=None,
                                help="specify alternate temporary directory")
        global_grp.add_argument("-t", "--threads", action="store",
                                dest="threads", default=4, type=int,
                                help="Number of threads to use")
        global_grp.add_argument("-v", "--verbose", action="count",
                                dest="verbosity", default=0,
                                help="increase verbosity")

        global_grp.add_argument('-z', '--compression-type',
                                dest="compression_type",
                                choices=['auto', 'gzip', 'xz'],
                                help="compression technology to use")

        # Group to make tarball encryption (via GPG/password) exclusive
        encrypt_grp = global_grp.add_mutually_exclusive_group()
        encrypt_grp.add_argument("--encrypt", default=False,
                                 action="store_true",
                                 help=("Encrypt the archive, either prompting "
                                       "for a password/key or referencing "
                                       "an environment variable"))
        encrypt_grp.add_argument("--encrypt-key",
                                 help="Encrypt the archive using a GPG "
                                      "key-pair")
        encrypt_grp.add_argument("--encrypt-pass",
                                 help="Encrypt the archive using a password")

    def _init_component(self):
        """Determine which component has been requested by the user, and then
        initialize that component.
        """
        _com = self.args.component
        if _com not in self._components.keys():
            print("Unknown subcommand '%s' specified" % _com)
        try:
            _to_load = self._components[_com][0]
            if _to_load.root_required and not os.getuid() == 0:
                raise Exception("Component must be run with root privileges")
            self._component = _to_load(self.parser, self.args, self.cmdline)

        except Exception as err:
            print("Could not initialize '%s': %s" % (_com, err))
            if self.args.debug:
                raise err
            sys.exit(1)

    def execute(self):
        self._component.execute()

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class TigerVNC(Plugin, RedHatPlugin):
    """
    This plugin gathers information for VNC servers provided by the tigervnc
    package. This is explicitly for server-side collections, not clients.

    By default, this plugin will capture the contents of /etc/tigervnc, which
    may include usernames. If usernames are sensitive information for end
    users of sos, consider using the `--clean` option to obfuscate these
    names.
    """

    short_desc = 'TigerVNC server configuration'
    plugin_name = 'tigervnc'
    packages = ('tigervnc-server',)

    def setup(self):
        self.add_copy_spec('/etc/tigervnc/')

        # service names are 'vncserver@$port' where $port is :1,, :2, etc...
        # however they are not reported via list-unit-files, only list-units
        vncs = self.exec_cmd(
            'systemctl list-units --type=service --no-legend vncserver*'
        )
        if vncs['status'] == 0:
            for serv in vncs['output'].splitlines():
                vnc = serv.split()
                if not vnc:
                    continue
                self.add_service_status(vnc[0])
                self.add_journal(vnc[0])

        self.add_cmd_output('vncserver -list')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Lustre(Plugin, RedHatPlugin):

    short_desc = 'Lustre filesystem'

    plugin_name = 'lustre'
    profiles = ('storage', 'network', 'cluster', )
    packages = ('lustre', 'lustre-client', )

    def get_params(self, name, param_list):
        """Use lctl get_param to collect a selection of parameters into a
            file.

        """
        self.add_cmd_output("lctl get_param %s" % " ".join(param_list),
                            suggest_filename="params-%s" % name,
                            stderr=False)

    def setup(self):
        self.add_cmd_output([
            "lctl debug_kernel",
            "lctl device_list",
            "lctl list_nids",
            "lctl route_list",
            "lnetctl net show -v"
        ])

        self.get_params("basic", ["version", "health_check", "debug"])
        self.get_params("lnet", ["peers", "routes", "routers", "nis"])
        self.get_params("ldlm-states", ["*.*.state"])
        self.get_params("jobid", ["jobid_name", "jobid_var"])

        # Client Specific
        self.add_cmd_output([
                "lfs df",
                "lfs df -i"
        ])

        # Server Specific
        self.get_params("osd", ["osd-*.*.{mntdev,files*," +
                                "kbytes*,blocksize,brw_stats}"])
        self.get_params("quota", ["osd-*.*.quota_slave." +
                                  "{info,limit_*,acct_*}"])

        # Grab emergency ring buffer dumps
        if self.get_option("all_logs"):
            self.add_copy_spec("/tmp/lustre-log.*")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Dracut(Plugin, RedHatPlugin):

    short_desc = 'Dracut initramfs generator'

    plugin_name = "dracut"
    packages = ("dracut",)
    profiles = ("boot",)

    def setup(self):
        self.add_copy_spec([
            "/etc/dracut.conf",
            "/etc/dracut.conf.d"
        ])

        self.add_cmd_output([
            "dracut --list-modules",
            "dracut --print-cmdline"
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class SELinux(Plugin, RedHatPlugin):

    short_desc = 'SELinux access control'

    plugin_name = 'selinux'
    profiles = ('container', 'system', 'security', 'openshift')

    option_list = [
        PluginOpt('fixfiles', default=False,
                  desc='collect incorrect file context labels')
    ]
    packages = ('libselinux',)

    def setup(self):
        self.add_copy_spec([
            '/etc/sestatus.conf',
            '/etc/selinux'
        ])
        # capture this with a higher log limit since #2035 may limit this
        # collection
        self.add_copy_spec('/var/lib/selinux', sizelimit=50)
        self.add_cmd_output('sestatus')

        state = self.exec_cmd('getenforce')['output']
        if state != 'Disabled':
            self.add_cmd_output([
                'ps auxZww',
                'sestatus -v',
                'sestatus -b',
                'selinuxdefcon root',
                'selinuxconlist root',
                'selinuxexeccon /bin/passwd',
                'semanage -o'  # deprecated, may disappear at some point
            ])

            subcmds = [
                'fcontext',
                'user',
                'port',
                'login',
                'node',
                'interface',
                'module'
            ]

            for subcmd in subcmds:
                self.add_cmd_output("semanage %s -l" % subcmd)

            if self.get_option('fixfiles'):
                self.add_cmd_output("restorecon -Rvn /", stderr=False,
                                    priority=100)

# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.cleaner.archives import SoSObfuscationArchive

import os
import tarfile


class DataDirArchive(SoSObfuscationArchive):
    """A plain directory on the filesystem that is not directly associated with
    any known or supported collection utility
    """

    type_name = 'data_dir'
    description = 'unassociated directory'

    @classmethod
    def check_is_type(cls, arc_path):
        return os.path.isdir(arc_path)

    def set_archive_root(self):
        return os.path.abspath(self.archive_path)


class TarballArchive(SoSObfuscationArchive):
    """A generic tar archive that is not associated with any known or supported
    collection utility
    """

    type_name = 'tarball'
    description = 'unassociated tarball'

    @classmethod
    def check_is_type(cls, arc_path):
        try:
            return tarfile.is_tarfile(arc_path)
        except Exception:
            return False

    def set_archive_root(self):
        if self.tarobj.firstmember.isdir():
            return self.tarobj.firstmember.name
        return ''



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos_tests import StageTwoReportTest


class OptionsFromConfigTest(StageTwoReportTest):
    """Ensure that we handle options specified in sos.conf properly

    :avocado: tags=stagetwo
    """

    files = [('options_tests_sos.conf', '/etc/sos/sos.conf')]
    sos_cmd = '-v '

    def test_case_id_from_config(self):
        self.assertTrue('8675309' in self.archive)

    def test_plugins_skipped_from_config(self):
        self.assertPluginNotIncluded(['networking', 'logs'])

    def test_plugopts_logged_from_config(self):
        self.assertSosLogContains(
            "Set kernel plugin option to \(name=with-timer, desc='gather /proc/timer\* statistics', value=True, default=False\)"
        )
        self.assertSosLogContains(
            "Set kernel plugin option to \(name=trace, desc='gather /sys/kernel/debug/tracing/trace file', value=True, default=False\)"
        )

    def test_disabled_plugopts_not_loaded(self):
        self.assertSosLogNotContains("Set networking plugin option to")

    def test_plugopts_actually_set(self):
        self.assertFileCollected('sys/kernel/debug/tracing/trace')

    def test_effective_options_logged_correctly(self):
        self.assertSosLogContains(
            "effective options now: --batch --case-id 8675309 --plugopts kernel.with-timer=on,kernel.trace=yes --skip-plugins networking,logs"
        )



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from argparse import Action
from configparser import (ConfigParser, ParsingError, Error,
                          DuplicateOptionError)


def _is_seq(val):
    """Return true if val is an instance of a known sequence type.
    """
    val_type = type(val)
    return val_type is list or val_type is tuple


def str_to_bool(val):
    _val = val.lower()
    if _val in ['true', 'on', 'yes']:
        return True
    elif _val in ['false', 'off', 'no']:
        return False
    else:
        return None


class SoSOptions():

    def _merge_opt(self, opt, src, is_default):
        def _unset(val):
            return (val == "" or val is None)

        if hasattr(src, opt):
            newvalue = getattr(src, opt)
            oldvalue = getattr(self, opt)
            # overwrite value iff:
            # - we replace unset option by a real value
            # - new default is set, or
            # - non-sequential variable keeps its default value
            if (_unset(oldvalue) and not _unset(newvalue)) or \
               is_default or \
               ((opt not in self._nondefault) and (not _is_seq(newvalue))):
                # Overwrite atomic values
                setattr(self, opt, newvalue)
                if is_default:
                    self._nondefault.discard(opt)
                else:
                    self._nondefault.add(opt)
            elif _is_seq(newvalue):
                # Concatenate sequence types
                setattr(self, opt, newvalue + oldvalue)

    def _merge_opts(self, src, is_default):
        if not isinstance(src, dict):
            src = vars(src)
        for arg in self.arg_names:
            self._merge_opt(arg, src, is_default)

    def __str(self, quote=False, sep=" ", prefix="", suffix=""):
        """Format a SoSOptions object as a human or machine readable string.

            :param quote: quote option values
            :param sep: list separator string
            :param prefix: arbitrary prefix string
            :param suffix: arbitrary suffix string
            :param literal: print values as Python literals
        """
        args = prefix
        arg_fmt = "=%s"
        for arg in self.arg_names:
            args += arg + arg_fmt + sep
        args.strip(sep)

        vals = [getattr(self, arg) for arg in self.arg_names]
        if not quote:
            # Convert Python source notation for sequences into plain strings
            vals = [",".join(v) if _is_seq(v) else v for v in vals]
        else:
            # Only quote strings if quote=False
            vals = ["'%s'" % v if isinstance(v, str) else v for v in vals]

        return (args % tuple(vals)).strip(sep) + suffix

    def __str__(self):
        return self.__str()

    def __repr__(self):
        return self.__str(quote=True, sep=", ", prefix="SoSOptions(",
                          suffix=")")

    def __init__(self, arg_defaults={}, **kwargs):
        """Initialise a new ``SoSOptions`` object from keyword arguments.

            Initialises the new object with values taken from keyword
            arguments matching the names of ``SoSOptions`` attributes.

            A ``ValueError`` is raised is any of the supplied keyword
            arguments does not correspond to a known ``SoSOptions`
            attribute name.

            :param *kwargs: a list of ``SoSOptions`` keyword args.
            :returns: the new ``SoSOptions`` object.
        """
        self.arg_defaults = arg_defaults
        self.arg_names = list(arg_defaults.keys())
        self._nondefault = set()
        # first load the defaults, if supplied
        for arg in self.arg_defaults:
            setattr(self, arg, self.arg_defaults[arg])
        # next, load any kwargs
        for arg in kwargs.keys():
            self.arg_names.append(arg)
            setattr(self, arg, kwargs[arg])

    @classmethod
    def from_args(cls, args, arg_defaults={}):
        """Initialise a new SoSOptions object from a ``Namespace``
            obtained by parsing command line arguments.

            :param args: parsed command line arguments
            :returns: an initialised SoSOptions object
            :returntype: SoSOptions
        """
        opts = SoSOptions(**vars(args), arg_defaults=arg_defaults)
        opts._merge_opts(args, True)
        return opts

    @classmethod
    def _opt_to_args(cls, opt, val):
        """Convert a named option and optional value to command line
            argument notation, correctly handling options that take
            no value or that have special representations (e.g. verify
            and verbose).
        """
        no_value = (
            "alloptions", "allow-system-changes", "all-logs", "batch", "build",
            "debug", "experimental", "list-plugins", "list-presets",
            "list-profiles", "no-report", "no-env-vars", "quiet", "verify"
        )
        count = ("verbose",)
        if opt in no_value:
            return ["--%s" % opt]
        if opt in count:
            return ["--%s" % opt for d in range(0, int(val))]
        return ["--" + opt + "=" + val]

    def _convert_to_type(self, key, val, conf):
        """Ensure that the value read from a config file is the proper type
        for consumption by the component, as defined by arg_defaults.

        Params:
            :param key:         The key in arg_defaults we need to match the
                                type of
            :param val:         The value to be converted to a particular type
            :param conf:        File values are being loaded from
        """
        if isinstance(self.arg_defaults[key], type(val)):
            return val
        if isinstance(self.arg_defaults[key], list):
            return [v for v in val.split(',')]
        if isinstance(self.arg_defaults[key], bool):
            val = str_to_bool(val)
            if val is None:
                raise Exception(
                    "Value of '%s' in %s must be True or False or analagous"
                    % (key, conf))
            else:
                return val
        if isinstance(self.arg_defaults[key], int):
            try:
                return int(val)
            except ValueError:
                raise Exception("Value of '%s' in %s must be integer"
                                % (key, conf))
        return val

    def update_from_conf(self, config_file, component):
        """Read the provided config_file and update options from that.

        Positional arguments:

            :param config_file:             Filepath to the config file
            :param component:               Which component (section) to load
        """

        def _update_from_section(section, config):
            if config.has_section(section):
                odict = dict(config.items(section))
                # handle verbose explicitly
                if 'verbose' in odict.keys():
                    odict['verbosity'] = int(odict.pop('verbose'))
                # convert options names
                # unify some of them if multiple variants of the
                # cmdoption exist
                rename_opts = {
                    'name': 'label',
                    'plugin_option': 'plugopts',
                    'profile': 'profiles'
                }
                for key in list(odict):
                    if '-' in key:
                        odict[key.replace('-', '_')] = odict.pop(key)
                    if key in rename_opts:
                        odict[rename_opts[key]] = odict.pop(key)
                # set the values according to the config file
                for key, val in odict.items():
                    # most option values do not tolerate spaces, special
                    # exception however for --keywords which we do want to
                    # support phrases, and thus spaces, for
                    if isinstance(val, str) and key != 'keywords':
                        val = val.replace(' ', '')
                    if key not in self.arg_defaults:
                        # read an option that is not loaded by the current
                        # SoSComponent
                        print("Unknown option '%s' in section '%s'"
                              % (key, section))
                        continue
                    val = self._convert_to_type(key, val, config_file)
                    setattr(self, key, val)

        config = ConfigParser()
        try:
            try:
                with open(config_file) as f:
                    config.readfp(f)
            except DuplicateOptionError as err:
                raise exit("Duplicate option '%s' in section '%s' in file %s"
                           % (err.option, err.section, config_file))
            except (ParsingError, Error):
                raise exit('Failed to parse configuration file %s'
                           % config_file)
        except (OSError, IOError) as e:
            print(
                'WARNING: Unable to read configuration file %s : %s'
                % (config_file, e.args[1])
            )

        _update_from_section("global", config)
        _update_from_section(component, config)
        if config.has_section("plugin_options") and hasattr(self, 'plugopts'):
            for key, val in config.items("plugin_options"):
                if not key.split('.')[0] in self.skip_plugins:
                    self.plugopts.append(key + '=' + val)

    def merge(self, src, skip_default=True):
        """Merge another set of ``SoSOptions`` into this object.

            Merge two ``SoSOptions`` objects by setting unset or default
            values to their value in the ``src`` object.

            :param src: the ``SoSOptions`` object to copy from
            :param is_default: ``True`` if new default values are to be set.
        """
        for arg in self.arg_names:
            if not hasattr(src, arg):
                continue
            if getattr(src, arg) is not None or not skip_default:
                self._merge_opt(arg, src, False)

    def dict(self, preset_filter=True):
        """Return this ``SoSOptions`` option values as a dictionary of
            argument name to value mappings.

            :returns: a name:value dictionary of option values.
        """
        odict = {}
        for arg in self.arg_names:
            value = getattr(self, arg)
            # Do not attempt to store preset option values in presets
            if preset_filter:
                if arg in ('add_preset', 'del_preset', 'desc', 'note'):
                    value = None
            odict[arg] = value
        return odict

    def to_args(self):
        """Return command arguments for this object.

            Return a list of the non-default options of this ``SoSOptions``
            object in ``sosreport`` command line argument notation:

                ``["--all-logs", "-vvv"]``

        """
        def has_value(name, value):
            """ Test for non-null option values.
            """
            null_values = ("False", "None", "[]", '""', "''", "0")
            if not value or value in null_values:
                return False
            if name == 'plugopts' and value:
                return True
            if name in self.arg_defaults:
                if str(value) == str(self.arg_defaults[name]):
                    return False
            return True

        def filter_opt(name, value):
            """ Filter out preset and null-valued options.
            """
            if name in ("add_preset", "del_preset", "desc", "note"):
                return False
            # Exception list for options that still need to be reported when 0
            if name in ['log_size', 'plugin_timeout', 'cmd_timeout'] \
               and value == 0:
                return True
            return has_value(name, value)

        def argify(name, value):
            """ Convert sos option notation to command line arguments.
            """
            # Handle --verbosity specially
            if name.startswith("verbosity"):
                arg = "-" + int(value) * "v"
                return arg

            name = name.replace("_", "-")

            value = ",".join(value) if _is_seq(value) else value

            if value is not True:
                opt = "%s %s" % (name, value)
            else:
                opt = name

            arg = "--" + opt if len(opt) > 1 else "-" + opt
            return arg

        opt_items = sorted(self.dict().items(), key=lambda x: x[0])
        return [argify(n, v) for (n, v) in opt_items if filter_opt(n, v)]


class SosListOption(Action):

    """Allow to specify comma delimited list of plugins"""

    def __call__(self, parser, namespace, values, option_string=None):
        items = [opt for opt in values.split(',')]
        if getattr(namespace, self.dest):
            items += getattr(namespace, self.dest)
        setattr(namespace, self.dest, items)


class ClusterOption():
    """Used to store/manipulate options for cluster profiles."""

    def __init__(self, name, value, opt_type, cluster, description=None):
        self.name = name
        self.value = value
        self.opt_type = opt_type
        self.cluster = cluster
        self.description = description



# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin
import re


class Kpatch(Plugin, RedHatPlugin):

    short_desc = 'Kpatch information'

    plugin_name = 'kpatch'

    packages = ('kpatch',)

    def setup(self):
        kpatch_list = self.collect_cmd_output("kpatch list")
        if not kpatch_list['status'] == 0:
            return
        kpatches = kpatch_list['output'].splitlines()
        for patch in kpatches:
            if not re.match(r"^kpatch-.*\(.*\)", patch):
                continue
            (module, version) = patch.split()
            self.add_cmd_output("kpatch info " + module)


# vim: set et ts=4 sw=4 :



# Copyright (C) 2010 Red Hat, Inc., Tomas Smetana <tsmetana@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Abrt(Plugin, RedHatPlugin):

    short_desc = 'Automatic Bug Reporting Tool'

    plugin_name = "abrt"
    profiles = ('system', 'debug')
    packages = ('abrt-cli', 'abrt-gui', 'abrt')
    files = ('/var/spool/abrt',)

    option_list = [
        PluginOpt("detailed", default=False,
                  desc="collect detailed information for every report")
    ]

    def setup(self):
        self.add_cmd_output("abrt-cli status",
                            tags=["abrt_status", "insights_abrt_status_bare"])
        abrt_list = self.collect_cmd_output("abrt-cli list")
        if self.get_option("detailed") and abrt_list['status'] == 0:
            for line in abrt_list["output"].splitlines():
                if line.startswith("Directory"):
                    self.add_cmd_output("abrt-cli info -d '%s'"
                                        % line.split()[1])

        self.add_copy_spec([
            "/etc/abrt/abrt.conf",
            "/etc/abrt/abrt-action-save-package-data.conf",
            "/etc/abrt/plugins"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Pavel Moravec <pmoravec@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Ptp(Plugin, IndependentPlugin):

    short_desc = 'Precision time protocol'

    plugin_name = "ptp"
    profiles = ('system', 'services')

    packages = ('linuxptp',)

    def setup(self):
        self.add_copy_spec([
            "/etc/ptp4l.conf",
            "/etc/timemaster.conf",
            "/sys/class/ptp"
        ])

# vim: et ts=4 sw=4



# Copyright (C) 2020 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.options import SoSOptions
from sos.presets import PresetDefaults


RHEL_RELEASE_STR = "Red Hat Enterprise Linux"

_opts_verify = SoSOptions(verify=True)
_opts_all_logs = SoSOptions(all_logs=True)
_opts_all_logs_verify = SoSOptions(all_logs=True, verify=True)
_cb_profiles = ['boot', 'storage', 'system']
_cb_plugopts = ['boot.all-images=on', 'rpm.rpmva=on', 'rpm.rpmdb=on']


RHV = "rhv"
RHV_DESC = "Red Hat Virtualization"

RHEL = "rhel"
RHEL_DESC = RHEL_RELEASE_STR

RHOSP = "rhosp"
RHOSP_DESC = "Red Hat OpenStack Platform"
RHOSP_OPTS = SoSOptions(plugopts=[
                             'process.lsof=off',
                             'networking.ethtool_namespaces=False',
                             'networking.namespaces=200'])

RHOCP = "ocp"
RHOCP_DESC = "OpenShift Container Platform by Red Hat"
RHOCP_OPTS = SoSOptions(
    verify=True, skip_plugins=['cgroups'], container_runtime='crio',
    no_report=True, log_size=100,
    plugopts=[
        'crio.timeout=600',
        'networking.timeout=600',
        'networking.ethtool_namespaces=False',
        'networking.namespaces=200'
    ])

RH_CFME = "cfme"
RH_CFME_DESC = "Red Hat CloudForms"

RH_SATELLITE = "satellite"
RH_SATELLITE_DESC = "Red Hat Satellite"
SAT_OPTS = SoSOptions(log_size=100, plugopts=['apache.log=on'])

CB = "cantboot"
CB_DESC = "For use when normal system startup fails"
CB_OPTS = SoSOptions(
            verify=True, all_logs=True, profiles=_cb_profiles,
            plugopts=_cb_plugopts
          )
CB_NOTE = ("Data collection will be limited to a boot-affecting scope")

NOTE_SIZE = "This preset may increase report size"
NOTE_TIME = "This preset may increase report run time"
NOTE_SIZE_TIME = "This preset may increase report size and run time"

RHEL_PRESETS = {
    RHV: PresetDefaults(name=RHV, desc=RHV_DESC, note=NOTE_TIME,
                        opts=_opts_verify),
    RHEL: PresetDefaults(name=RHEL, desc=RHEL_DESC),
    RHOSP: PresetDefaults(name=RHOSP, desc=RHOSP_DESC, opts=RHOSP_OPTS),
    RHOCP: PresetDefaults(name=RHOCP, desc=RHOCP_DESC, note=NOTE_SIZE_TIME,
                          opts=RHOCP_OPTS),
    RH_CFME: PresetDefaults(name=RH_CFME, desc=RH_CFME_DESC, note=NOTE_TIME,
                            opts=_opts_verify),
    RH_SATELLITE: PresetDefaults(name=RH_SATELLITE, desc=RH_SATELLITE_DESC,
                                 note=NOTE_TIME, opts=SAT_OPTS),
    CB: PresetDefaults(name=CB, desc=CB_DESC, note=CB_NOTE, opts=CB_OPTS)
}


ATOMIC = "atomic"
ATOMIC_RELEASE_STR = "Atomic"
ATOMIC_DESC = "Red Hat Enterprise Linux Atomic Host"

ATOMIC_PRESETS = {
    ATOMIC: PresetDefaults(name=ATOMIC, desc=ATOMIC_DESC, note=NOTE_TIME,
                           opts=_opts_verify)
}


# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin
from re import match
import glob


class Peripety(Plugin, RedHatPlugin):

    short_desc = 'Peripety Storage Event Monitor'
    plugin_name = 'peripety'
    packages = ('peripety',)
    services = ('peripetyd',)

    def setup(self):
        self.add_copy_spec('/etc/peripetyd.conf')

        forbid_reg = [
            'vd.*',
            'sr.*',
            'loop.*',
            'ram.*'
        ]

        disks = filter(lambda x: not any(match(reg, x) for reg in forbid_reg),
                       [d.split('/')[-1] for d in glob.glob('/sys/block/*')])

        for disk in disks:
            self.add_cmd_output([
                "prpt info %s" % disk,
                "prpt query --blk %s" % disk
            ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class vhostmd(Plugin, RedHatPlugin):

    short_desc = 'vhostmd virtualization metrics collection'

    plugin_name = 'vhostmd'
    profiles = ('sap', 'virt', 'system')

    packages = ('virt-what',)

    def setup(self):
        vw = self.collect_cmd_output("virt-what")['output'].splitlines()

        if not vw:
            return

        if "vmware" in vw or "kvm" in vw or "xen" in vw:
            if self.is_installed("vm-dump-metrics"):
                # if vm-dump-metrics is installed use it
                self.add_cmd_output("vm-dump-metrics",
                                    suggest_filename="virt_metrics")
            else:
                # otherwise use the raw vhostmd disk presented (256k size)
                sysblock = "/sys/block"
                if not self.path_isdir(sysblock):
                    return
                for disk in self.listdir(sysblock):
                    if "256K" in disk:
                        dev = disk.split()[0]
                        r = self.exec_cmd("dd if=/dev/%s bs=25 count=1" % dev)
                        if 'metric' in r['output']:
                            self.add_cmd_output(
                                "dd if=/dev/%s bs=256k count=1" % dev,
                                suggest_filename="virt_metrics"
                            )

# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class System(Plugin, IndependentPlugin):

    short_desc = 'core system information'

    plugin_name = "system"
    profiles = ('system', 'kernel')
    verify_packages = ('glibc', 'initscripts', 'zlib')

    def setup(self):
        self.add_copy_spec([
            "/proc/sys",
            "/etc/sysconfig",
            "/etc/default",
            "/etc/environment",
        ])

        # FIXME: provide a a long-term solution for #1299
        self.add_forbidden_path([
            "/proc/sys/net/ipv4/route/flush",
            "/proc/sys/net/ipv6/route/flush",
            "/proc/sys/net/ipv6/neigh/*/retrans_time",
            "/proc/sys/net/ipv6/neigh/*/base_reachable_time"
        ])

        # collect glibc tuning decisions
        self.add_cmd_output([
            "ld.so --help",
            "ld.so --list-diagnostics",
            "ld.so --list-tunables"
        ])


# vim: set et ts=4 sw=4 :



# Copyright (C) 2013 Red Hat, Inc.
# Copyright (C) 2017 Red Hat, Inc., Martin Schuppert <mschuppert@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin
import os


class OpenStackHeat(Plugin):

    short_desc = 'OpenStack Heat'
    plugin_name = "openstack_heat"
    profiles = ('openstack', 'openstack_controller')
    containers = ('.*heat_api',)
    var_puppet_gen = "/var/lib/config-data/puppet-generated/heat"
    service_name = "openstack-heat-api.service"

    def setup(self):

        # collect commands output only if the openstack-heat-api service
        # is running
        in_container = self.container_exists('.*heat_api')

        if self.is_service_running(self.service_name) or in_container:
            heat_config = ""
            # if containerized we need to pass the config to the cont.
            if in_container:
                heat_config = "--config-dir " + self.var_puppet_gen + \
                                "_api/etc/heat/"

            self.add_cmd_output(
                "heat-manage " + heat_config + " db_version",
                suggest_filename="heat_db_version"
            )

            vars_all = [p in os.environ for p in [
                        'OS_USERNAME', 'OS_PASSWORD']]

            vars_any = [p in os.environ for p in [
                        'OS_TENANT_NAME', 'OS_PROJECT_NAME']]

            if not (all(vars_all) and any(vars_any)):
                self.soslog.warning("Not all environment variables set. "
                                    "Source the environment file for the user "
                                    "intended to connect to the OpenStack "
                                    "environment.")
            else:
                self.add_cmd_output("openstack stack list")

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/heat/",
            ])
        else:
            self.add_copy_spec([
                "/var/log/heat/*.log",
            ])

        self.add_copy_spec([
            "/etc/heat/",
            self.var_puppet_gen + "/etc/heat/",
            self.var_puppet_gen + "/etc/my.cnf.d/tripleo.cnf",
            self.var_puppet_gen + "_api/etc/heat/",
            self.var_puppet_gen + "_api/etc/httpd/conf/",
            self.var_puppet_gen + "_api/etc/httpd/conf.d/",
            self.var_puppet_gen + "_api/etc/httpd/conf.modules.d/*.conf",
            self.var_puppet_gen + "_api/var/spool/cron/heat",
            self.var_puppet_gen + "_api_cfn/etc/heat/",
            self.var_puppet_gen + "_api_cfn/etc/httpd/conf/",
            self.var_puppet_gen + "_api_cfn/etc/httpd/conf.d/",
            self.var_puppet_gen + "_api_cfn/etc/httpd/conf.modules.d/*.conf",
            self.var_puppet_gen + "_api_cfn/var/spool/cron/heat",
        ])

    def apply_regex_sub(self, regexp, subst):
        self.do_path_regex_sub(
            "/etc/heat/*",
            regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/heat/*",
            regexp, subst
        )
        self.do_path_regex_sub(
            self.var_puppet_gen + "_api/etc/heat/*",
            regexp, subst
        )
        self.do_path_regex_sub(
            self.var_puppet_gen + "_api_cfn/etc/heat/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "admin_password", "memcache_secret_key", "password",
            "qpid_password", "rabbit_password", "stack_domain_admin_password",
            "transport_url"
        ]
        connection_keys = ["connection"]

        self.apply_regex_sub(
            r"((?m)^\s*(%s)\s*=\s*)(.*)" % "|".join(protect_keys),
            r"\1*********"
        )
        self.apply_regex_sub(
            r"((?m)^\s*(%s)\s*=\s*(.*)://(\w*):)(.*)(@(.*))" %
            "|".join(connection_keys),
            r"\1*********\6"
        )


class DebianHeat(OpenStackHeat, DebianPlugin, UbuntuPlugin):

    packages = (
        'heat-api',
        'heat-api-cfn',
        'heat-api-cloudwatch',
        'heat-common',
        'heat-engine',
        'python-heat',
        'python-heatclient'
    )
    service_name = 'heat-api.service'


class RedHatHeat(OpenStackHeat, RedHatPlugin):

    packages = ('openstack-selinux',)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Red Hat, Inc.

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class AtomicHost(Plugin, RedHatPlugin):

    short_desc = 'Atomic Host'

    plugin_name = "atomichost"
    profiles = ('container',)
    option_list = [
        PluginOpt("info", default=False,
                  desc="gather atomic info for each image")
    ]

    def check_enabled(self):
        return self.policy.in_container()

    def setup(self):
        self.add_cmd_output("atomic host status")

        if self.get_option('info'):
            # The 'docker images' command may include duplicate rows of
            # output (repeated "IMAGE ID" values). Use a set to filter
            # these out and only obtain 'docker info' data once per image
            # identifier.
            images = self.exec_cmd("docker images -q")
            for image in set(images['output'].splitlines()):
                self.add_cmd_output("atomic info {0}".format(image))

# vim: set et ts=4 sw=4 :



# Copyright (C) 2009 Red Hat, Inc., Joey Boggs <jboggs@redhat.com>
# Copyright (C) 2012 Rackspace US, Inc.,
#                    Justin Shepherd <jshepher@rackspace.com>
# Copyright (C) 2013 Red Hat, Inc., Flavio Percoco <fpercoco@redhat.com>
# Copyright (C) 2013 Red Hat, Inc., Jeremy Agee <jagee@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackCinder(Plugin):

    short_desc = 'OpenStack cinder'
    plugin_name = "openstack_cinder"
    profiles = ('openstack', 'openstack_controller')
    containers = ('.*cinder_api',)

    var_puppet_gen = "/var/lib/config-data/puppet-generated/cinder"

    def setup(self):
        self.add_forbidden_path('/etc/cinder/volumes')
        cinder_config = ""
        cinder_config_opt = "--config-dir %s/etc/cinder/"

        # check if either standalone (cinder-api) or httpd wsgi (cinder_wsgi)
        # is up and running
        cinder_process = ["cinder_wsgi", "cinder-wsgi", "cinder-api"]
        in_ps = False
        for process in cinder_process:
            in_ps = self.check_process_by_name(process)
            if in_ps:
                break

        in_container = self.container_exists('.*cinder_api')
        if in_container:
            cinder_config = cinder_config_opt % self.var_puppet_gen

        # collect commands output if the standalone, wsgi or container is up
        if in_ps or in_container:
            self.add_cmd_output(
                "cinder-manage " + cinder_config + " db version",
                suggest_filename="cinder_db_version"
            )

        self.add_forbidden_path('/etc/cinder/volumes')
        self.add_copy_spec([
            "/etc/cinder/",
            self.var_puppet_gen + "/etc/cinder/",
            self.var_puppet_gen + "/etc/httpd/conf/",
            self.var_puppet_gen + "/etc/httpd/conf.d/",
            self.var_puppet_gen + "/etc/httpd/conf.modules.d/*.conf",
            self.var_puppet_gen + "/etc/my.cnf.d/tripleo.cnf",
            self.var_puppet_gen + "/etc/sysconfig/",
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/cinder/",
                "/var/log/httpd/cinder*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/cinder/*.log",
                "/var/log/httpd/cinder*.log",
            ])

    def apply_regex_sub(self, regexp, subst):
        self.do_path_regex_sub("/etc/cinder/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/cinder/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "admin_password", "backup_tsm_password", "chap_password",
            "nas_password", "cisco_fc_fabric_password", "coraid_password",
            "eqlx_chap_password", "fc_fabric_password",
            "hitachi_auth_password", "hitachi_horcm_password",
            "hp3par_password", "hplefthand_password", "memcache_secret_key",
            "netapp_password", "netapp_sa_password", "nexenta_password",
            "password", "qpid_password", "rabbit_password", "san_password",
            "ssl_key_password", "vmware_host_password", "zadara_password",
            "zfssa_initiator_password", "hmac_keys", "zfssa_target_password",
            "os_privileged_user_password", "transport_url"
        ]
        connection_keys = ["connection"]

        self.apply_regex_sub(
            r"((?m)^\s*(%s)\s*=\s*)(.*)" % "|".join(protect_keys),
            r"\1*********"
        )
        self.apply_regex_sub(
            r"((?m)^\s*(%s)\s*=\s*(.*)://(\w*):)(.*)(@(.*))" %
            "|".join(connection_keys),
            r"\1*********\6"
        )


class DebianCinder(OpenStackCinder, DebianPlugin, UbuntuPlugin):

    cinder = False
    packages = (
        'cinder-api',
        'cinder-backup',
        'cinder-common',
        'cinder-scheduler',
        'cinder-volume',
        'python-cinder',
        'python-cinderclient'
    )

    def setup(self):
        super(DebianCinder, self).setup()
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/apache/cinder*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/apache/cinder*.log",
            ])


class RedHatCinder(OpenStackCinder, RedHatPlugin):

    cinder = False
    packages = ('openstack-selinux',)

    def setup(self):
        super(RedHatCinder, self).setup()
        self.add_copy_spec(["/etc/sudoers.d/cinder"])


# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Stratis(Plugin, RedHatPlugin):

    short_desc = 'Stratis Storage'
    plugin_name = 'stratis'
    packages = ('stratis-cli', 'stratisd')
    services = ('stratisd',)
    profiles = ('storage',)

    def setup(self):
        subcmds = [
            'pool list',
            'pool list --stopped',
            'filesystem list',
            'blockdev list',
            'key list',
            'daemon version',
            'report engine_state_report',
            'report managed_objects_report',
            'report stopped_pools',
            '--version',
        ]

        self.add_cmd_output(["stratis %s" % subcmd for subcmd in subcmds])

# vim: set et ts=4 sw=4 :
